@@ -19,7 +19,9 @@ public class Log {
      */
     public static void log(int errorLevel, String message, Throwable e) {
         System.err.println(message);
-        e.printStackTrace();
+        if(e != null){
+            e.printStackTrace();
+        }
         try {
             
 	        Status s = new Status(errorLevel, CorePlugin.getPluginID(), errorLevel, message, e);
@@ -33,4 +35,8 @@ public class Log {
         log(IStatus.ERROR, e.getMessage() != null ? e.getMessage() : "No message gotten.", e);
     }
 
+    public static void log(String msg) {
+        log(IStatus.ERROR, msg, new RuntimeException(msg));
+    }
+
 }
