@@ -1,31 +1,53 @@
-import threading
 import traceback
 import weakref
-from pydevd_comm import *
+from pydevd_comm import * #@UnusedWildImport
+import pydevd_io
 
 DONT_TRACE = ('pydevd.py' , 'threading.py', 'pydevd_vars.py', 'pydevd_comm.py')
 
 connected = False
+bufferStdOutToServer = False
+bufferStdErrToServer = False
 
 #this is because jython does not have staticmethods
 PyDBCtx_threadToCtx = {}
 PyDBCtx_Lock = threading.RLock()
+PyDBCtx_Threads = {}#weakref.WeakValueDictionary()
 
+PyDBCtx_UseLocks = False #I don't know why jython halts when using this synchronization...
+PyDBCtxObjUseLocks = True
+PyDBUseLocks = True
+
+def PyDBCtx_LockAcquire():
+    if PyDBCtx_UseLocks:
+        PyDBCtx_Lock.acquire()
+    return True
+    
+def PyDBCtx_LockRelease():
+    if PyDBCtx_UseLocks:
+        PyDBCtx_Lock.release()
+    return True
+    
 def PyDBCtx_GetCtxs():
-    PyDBCtx_Lock.acquire()
+    PyDBCtx_LockAcquire()
     try:
         ret = []
         for v in PyDBCtx_threadToCtx.values():
             ret += v.values()
     finally:
-        PyDBCtx_Lock.release()
+        PyDBCtx_LockRelease()
     return ret
 
 def PyDBCtx_GetCtx(frame, currThread):
-    PyDBCtx_Lock.acquire()
+    PyDBCtx_LockAcquire()
     try:
-        #we create a context for each thread
+        #keep the last frame for each thread (previously we were keeping all the frames, in the context,
+        #but that was a bad thing, as things in the frames did not die).
         threadId = id(currThread)
+        PyDBCtx_Threads[threadId] = weakref.ref(currThread)
+        currThread._lastFrame = frame
+        
+        #we create a context for each thread
         ctxs = PyDBCtx_threadToCtx.get(threadId)
         if ctxs is None:
             ctxs = {}
@@ -38,21 +60,29 @@ def PyDBCtx_GetCtx(frame, currThread):
             ctx = PyDBCtx(frame, threadId)
             ctxs[key] = ctx
     finally:
-        PyDBCtx_Lock.release()
+        PyDBCtx_LockRelease()
 
     return ctx
 
 def PyDBCtx_SetTraceForAllFileCtxs(f):
-    PyDBCtx_Lock.acquire()
+    PyDBCtx_LockAcquire()
     try:
         g = GetGlobalDebugger()
         if g:
             for ctx in PyDBCtx_GetCtxs():
                 if ctx.filename == f:
-                    ctx.frame.f_trace = g.trace_dispatch
+                    thread = PyDBCtx_Threads.get(ctx.thread_id)()#it is a weakref
+                    if thread is not None:
+                        frame = thread._lastFrame
+                        while frame:
+                            currFile = NormFile(frame.f_code.co_filename)
+                            if f == currFile:
+                                frame.f_trace = g.trace_dispatch
+                            frame = frame.f_back
     finally:
-        PyDBCtx_Lock.release()
-                
+        PyDBCtx_LockRelease()
+
+
 class PyDBCtx:
     '''This class is used to keep track of the contexts we pass through (acting as a cache for them).
     '''
@@ -60,18 +90,42 @@ class PyDBCtx:
     def __init__(self, frame, threadId):
         self.filename = NormFile(frame.f_code.co_filename)
         self.base = os.path.basename( self.filename )
-        self.frame = frame
         self.thread_id = threadId
         self.lock = threading.RLock()
         
     def __str__(self):
-        return 'PyDBCtx [%s %s %s]' % (self.base, self.thread_id, self.frame)
+        return 'PyDBCtx [%s %s %s]' % (self.base, self.thread_id)
     
     def acquire(self):
-        self.lock.acquire()
+        if PyDBCtxObjUseLocks:
+            self.lock.acquire()
+        return True
         
     def release(self):
-        self.lock.release()
+        if PyDBCtxObjUseLocks:
+            self.lock.release()
+        return True
+        
+class PyDBCommandThread(PyDBDaemonThread):
+    
+    def __init__(self, pyDb):
+        PyDBDaemonThread.__init__(self)
+        self.pyDb = pyDb
+
+    def run(self):
+        time.sleep(5) #this one will only start later on (because otherwise we may not have any non-daemon threads
+        sys.settrace(None) # no debugging on this thread
+        try:
+            while not self.killReceived:
+                try:
+                    self.pyDb.processInternalCommands()
+                except:
+                    if pydevd_trace >= 0: print 'Finishing debug communication...(2)'
+                time.sleep(0.5)
+        except:
+            #only got this error in interpreter shutdown
+            if pydevd_trace >= 0: print 'Finishing debug communication... (3)'
+            
 
 class PyDBAdditionalThreadInfo:
     def __init__(self):
@@ -79,6 +133,8 @@ class PyDBAdditionalThreadInfo:
         self.pydev_step_stop = None
         self.pydev_step_cmd = None
         self.pydev_notify_kill = False
+        self.pydev_stop_on_return_count_1 = False
+        self.pydev_return_call_count = 0
 
 #---------------------------------------------------------------------------------------- THIS IS THE DEBUGGER
 
@@ -109,49 +165,36 @@ class PyDB:
         self.breakpoints = {}
         self.readyToRun = False
         self.lock = threading.RLock()
-
+        
+    def acquire(self):
+        if PyDBUseLocks:
+            self.lock.acquire()
+        return True
+    
+    def release(self):
+        if PyDBUseLocks:
+            self.lock.release()
+        return True
+        
     def initializeNetwork(self, sock):
         try:
             sock.settimeout(None) # infinite, no timeouts from now on - jython does not have it
         except:
             pass
         self.writer = WriterThread(sock)
-        self.writer.start()
         self.reader = ReaderThread(sock)
+        self.writer.start()
         self.reader.start()        
-        time.sleep(0.1) # give threads time to start        
         
-    def startServer(self, port):
-        """ binds to a port, waits for the debugger to connect """
-        s = socket(AF_INET, SOCK_STREAM)
-        s.bind(('', port))
-        s.listen(1)
-        newSock, addr = s.accept()
-        self.initializeNetwork(newSock)
-
-    def startClient(self, host, port):
-        """ connects to a host/port """
-        pydevd_log(1, "Connecting to " + host + ":" + str(port))
-        try:
-            s = socket(AF_INET, SOCK_STREAM);
-            try:
-                s.settimeout(10) # seconds - jython does not have it
-            except:
-                pass
-
-            s.connect((host, port))
-            pydevd_log(1, "Connected.")
-            self.initializeNetwork(s)
-        except:
-            print "server timed out after 10 seconds, could not connect to " + host + ":" + str(port)
-            print "Exiting. Bye!"
-            sys.exit(1)
+        time.sleep(0.1) # give threads time to start        
 
     def connect(self, host, port):
         if host:
-            self.startClient(host, port)
+            s = startClient(host, port)
         else:
-            self.startServer(port)
+            s = startServer(port)
+            
+        self.initializeNetwork(s)
         
 
     def getInternalQueue(self, thread_id):
@@ -176,22 +219,39 @@ class PyDB:
         return self.cmdQueue[thread_id]
         
     def postInternalCommand(self, int_cmd, thread_id):
-        """ if thread_id is *, post to all the threads (key)"""
-        self.lock.acquire()
+        """ if thread_id is *, post to all """
+        if thread_id == "*":
+            for k in self.cmdQueue.keys(): 
+                self.cmdQueue[k].put(int_cmd)
+                
+        else:
+            queue = self.getInternalQueue(thread_id)
+            queue.put(int_cmd)
+
+    def checkOutput(self, out, ctx):
+        '''Checks the output to see if we have to send some buffered output to the debug server
+        
+        @param out: sys.stdout or sys.stderr
+        @param ctx: the context indicating: 1=stdout and 2=stderr (to know the colors to write it)
+        '''
+        
         try:
-            if thread_id == "*":
-                for k in self.cmdQueue.keys(): 
-                    self.cmdQueue[k].put(int_cmd)
-                    
-            else:
-                queue = self.getInternalQueue(thread_id)
-                queue.put(int_cmd)
-        finally:
-            self.lock.release()
+            v = out.getvalue()
+            if v:
+                self.cmdFactory.makeIoMessage(v, ctx, self)
+        except:
+            traceback.print_exc()
+        
 
     def processInternalCommands(self):
-        self.lock.acquire()
+        self.acquire()
         try:
+            if bufferStdOutToServer:
+                self.checkOutput(sys.stdoutBuf,1)
+                    
+            if bufferStdErrToServer:
+                self.checkOutput(sys.stderrBuf,2)
+
             threads = threading.enumerate()
             foundNonPyDBDaemonThread = False
             for t in threads:
@@ -204,7 +264,7 @@ class PyDB:
                         while True:
                             int_cmd = queue.get(False)
                             if int_cmd.canBeExecutedBy(id(threading.currentThread())):
-                                pydevd_log(2, "processign internal command " + str(int_cmd))
+                                pydevd_log(2, "processing internal command " + str(int_cmd))
                                 int_cmd.doIt(self)
                             else:
                                 pydevd_log(2, "NOT processign internal command " + str(int_cmd))
@@ -226,11 +286,10 @@ class PyDB:
                 except:
                     sys.exit(0)
         finally:
-            self.lock.release()
+            self.release()
       
-        
     def processNetCommand(self, id, seq, text):
-        self.lock.acquire()
+        self.acquire()
         try:
             try:
                 cmd = None
@@ -350,24 +409,20 @@ class PyDB:
                     
             except Exception, e:
                 traceback.print_exc(e)
+                traceback.print_exc()
                 cmd = self.cmdFactory.makeErrorMessage(seq, "Unexpected exception in processNetCommand: %s\nInitial params: %s" % (str(e), (id, seq, text)))
                 self.writer.addCommand(cmd)
         finally:
-            self.lock.release()
+            self.release()
 
     def processThreadNotAlive(self, thread):
         """ if thread is not alive, cancel trace_dispatch processing """
-        self.lock.acquire()
-        try:
-            wasNotified = thread.additionalInfo.pydev_notify_kill
-                
-            if not wasNotified:
-                cmd = self.cmdFactory.makeThreadKilledMessage(id(thread))
-                self.writer.addCommand(cmd)
-                thread.additionalInfo.pydev_notify_kill = True
-        finally:
-            self.lock.release()
-
+        wasNotified = thread.additionalInfo.pydev_notify_kill
+            
+        if not wasNotified:
+            cmd = self.cmdFactory.makeThreadKilledMessage(id(thread))
+            self.writer.addCommand(cmd)
+            thread.additionalInfo.pydev_notify_kill = True
 
     def setSuspend(self, thread, stop_reason):
         thread.additionalInfo.pydev_state = PyDB.STATE_SUSPEND
@@ -385,7 +440,7 @@ class PyDB:
         info = thread.additionalInfo
         while info.pydev_state == PyDB.STATE_SUSPEND:            
             self.processInternalCommands()
-            time.sleep(0.1)
+            time.sleep(0.2)
             
         #process any stepping instructions 
         if info.pydev_step_cmd == CMD_STEP_INTO:
@@ -398,6 +453,8 @@ class PyDB:
                 info.pydev_step_stop = frame
                 
         elif info.pydev_step_cmd == CMD_STEP_RETURN:
+            info.pydev_stop_on_return_count_1 = True
+            info.pydev_return_call_count = 0
             info.pydev_step_stop = frame.f_back
  
         cmd = self.cmdFactory.makeThreadRunMessage(id(thread), info.pydev_step_cmd)
@@ -422,27 +479,28 @@ class PyDB:
         
         t = threading.currentThread()
         ctx = PyDBCtx_GetCtx(frame, t)
+        base = ctx.base
+
+        if base in DONT_TRACE: #we don't want to debug threading or anything related to pydevd
+            return None
+
+        # if thread is not alive, cancel trace_dispatch processing
+        if not t.isAlive():
+            self.processThreadNotAlive(t)
+            return None # suspend tracing
+        
+        try:
+            additionalInfo = t.additionalInfo
+        except AttributeError:
+            additionalInfo = PyDBAdditionalThreadInfo()
+            t.additionalInfo = additionalInfo
+            
+        filename = ctx.filename
+        released = False
         ctx.acquire()
         try:
-            base = ctx.base
-    
-            if base in DONT_TRACE: #we don't want to debug threading or anything related to pydevd
-                return None
     
     
-            # if thread is not alive, cancel trace_dispatch processing
-            if not t.isAlive():
-                self.processThreadNotAlive(t)
-                return None # suspend tracing
-            wasSuspended = False        
-    
-            try:
-                additionalInfo = t.additionalInfo
-            except AttributeError:
-                additionalInfo = PyDBAdditionalThreadInfo()
-                t.additionalInfo = additionalInfo
-                
-            filename = ctx.filename
             # Let's check to see if we are in a line that has a breakpoint. If we don't have a breakpoint, 
             # we will return nothing for the next trace
             #also, after we hit a breakpoint and go to some other debugging state, we have to force the set trace anyway,
@@ -484,7 +542,8 @@ class PyDB:
                     
                 # if thread has a suspend flag, we suspend with a busy wait
                 if additionalInfo.pydev_state == PyDB.STATE_SUSPEND:
-                    wasSuspended = True
+                    released = True
+                    ctx.release()
                     self.doWaitSuspend(t, frame, event, arg)
                     return self.trace_dispatch
                 
@@ -492,30 +551,58 @@ class PyDB:
                 traceback.print_exc()
                 raise
     
-            if not wasSuspended and (event == 'line' or event== 'return'):
-                #step handling. We stop when we hit the right frame
-                try:
-                    if additionalInfo.pydev_step_cmd == CMD_STEP_INTO:
-                        self.setSuspend(t, CMD_STEP_INTO)
-                        self.doWaitSuspend(t, frame, event, arg)      
-                        
+            #step handling. We stop when we hit the right frame
+            try:
+                if additionalInfo.pydev_step_cmd == CMD_STEP_INTO and event in ('line', 'return'):
+                    self.setSuspend(t, CMD_STEP_INTO)
+                    released = True
+                    ctx.release()
+                    self.doWaitSuspend(t, frame, event, arg)      
                     
-                    elif additionalInfo.pydev_step_cmd == CMD_STEP_OVER or additionalInfo.pydev_step_cmd == CMD_STEP_RETURN:
-                        if additionalInfo.pydev_step_stop == frame:
-                            self.setSuspend(t, additionalInfo.pydev_step_cmd)
-                            self.doWaitSuspend(t, frame, event, arg)
-                except:
-                    additionalInfo.pydev_step_cmd = None
-            
-            
-            #if we are quitting, let's stop the tracing
-            retVal = None
-            if not self.quitting:
-                retVal = self.trace_dispatch
-
-            return retVal
+                
+                elif additionalInfo.pydev_step_cmd == CMD_STEP_OVER and event in ('line', 'return'): 
+                    if additionalInfo.pydev_step_stop == frame:
+                        self.setSuspend(t, CMD_STEP_OVER)
+                        released = True
+                        ctx.release()
+                        self.doWaitSuspend(t, frame, event, arg)
+                    
+                    
+                elif additionalInfo.pydev_step_cmd == CMD_STEP_RETURN:
+                    if event == 'return':
+                        additionalInfo.pydev_return_call_count += 1
+                    elif event == 'call':
+                        additionalInfo.pydev_return_call_count -= 1
+                        
+                    if additionalInfo.pydev_stop_on_return_count_1 and additionalInfo.pydev_return_call_count == 1 \
+                        and additionalInfo.pydev_step_stop == frame.f_back  and event in ('line', 'return'):
+                        
+                        additionalInfo.pydev_return_call_count == 0
+                        additionalInfo.pydev_stop_on_return_count_1 = False
+                        self.setSuspend(t, CMD_STEP_RETURN)
+                        released = True
+                        ctx.release()
+                        self.doWaitSuspend(t, frame, event, arg)
+                        
+                    if additionalInfo.pydev_step_stop == frame and event in ('line', 'return'):
+                        self.setSuspend(t, CMD_STEP_RETURN)
+                        released = True
+                        ctx.release()
+                        self.doWaitSuspend(t, frame, event, arg)
+            except:
+                traceback.print_exc()
+                additionalInfo.pydev_step_cmd = None
         finally:
-            ctx.release()
+            if not released:
+                ctx.release()
+        
+        
+        #if we are quitting, let's stop the tracing
+        retVal = None
+        if not self.quitting:
+            retVal = self.trace_dispatch
+
+        return retVal
 
     def stopTracingFrame(self, frame):
         '''Removes the f_trace hook from the frame and returns None
@@ -561,15 +648,13 @@ class PyDB:
             del sys.path[0]
         
         #now, the local directory has to be added to the pythonpath
-        import os
         sys.path.insert(0, os.getcwd())
-        
         # for completness, we'll register the pydevd.reader & pydevd.writer threads
         net = NetCommand(str(CMD_THREAD_CREATE), 0, '<xml><thread name="pydevd.reader" id="-1"/></xml>')
         self.writer.addCommand(net)
         net = NetCommand(str(CMD_THREAD_CREATE), 0, '<xml><thread name="pydevd.writer" id="-1"/></xml>')
         self.writer.addCommand(net)
-        
+
         sys.settrace(self.trace_dispatch) 
         try:                     
             threading.settrace(self.trace_dispatch) # for all future threads           
@@ -579,6 +664,8 @@ class PyDB:
         while not self.readyToRun: 
             time.sleep(0.1) # busy wait until we receive run command
             
+        PyDBCommandThread(debugger).start()
+
         execfile(file, globals, locals) #execute the script
 
 
@@ -643,12 +730,20 @@ def SetTraceForParents(frame, dispatch_func):
         frame.f_trace = dispatch_func
         frame = frame.f_back
 
-def settrace(host='localhost'):
+def settrace(host='localhost', stdoutToServer = False, stderrToServer = False):
     '''
-    @param host: the user may specify another host, 
+    @param host: the user may specify another host, if the debug server is not in the same machine
+    @param stdoutToServer: when this is true, the stdout is passed to the debug server
+    @param stderrToServer: when this is true, the stderr is passed to the debug server
+    so that they are printed in its console and not in this process console.
     '''
     
     global connected
+    global bufferStdOutToServer
+    global bufferStdErrToServer
+    bufferStdOutToServer = stdoutToServer
+    bufferStdErrToServer = stderrToServer
+    
     if not connected :
         connected = True  
         
@@ -662,6 +757,14 @@ def settrace(host='localhost'):
         net = NetCommand(str(CMD_THREAD_CREATE), 0, '<xml><thread name="pydevd.writer" id="-1"/></xml>')
         debugger.writer.addCommand(net)
         
+        if bufferStdOutToServer:
+            sys.stdoutBuf = pydevd_io.IOBuf()
+            sys.stdout = pydevd_io.IORedirector(sys.stdout, sys.stdoutBuf)
+            
+        if bufferStdErrToServer:
+            sys.stderrBuf = pydevd_io.IOBuf()
+            sys.stderr = pydevd_io.IORedirector(sys.stderr, sys.stderrBuf)
+            
         SetTraceForParents(sys._getframe(), debugger.trace_dispatch)
         
         t = threading.currentThread()      
@@ -674,6 +777,7 @@ def settrace(host='localhost'):
         debugger.setSuspend(t, CMD_SET_BREAK)
         
         sys.settrace(debugger.trace_dispatch)
+        PyDBCommandThread(debugger).start()
     
 if __name__ == '__main__':
     print >>sys.stderr, "pydev debugger"
