@@ -1,4 +1,4 @@
-""" pydevd - a debugging daemon
+''' pydevd - a debugging daemon
 This is the daemon you launch for python remote debugging.
 
 Protocol:
@@ -15,38 +15,46 @@ each command has a format:
 
     Commands:
  
-    NUMBER   NAME            FROM*     ARGUMENTS           RESPONSE      NOTE
+    NUMBER   NAME                 FROM*     ARGUMENTS                     RESPONSE      NOTE
 100 series: program execution
-    101      RUN             RDB       -                   -
-    102      LIST_THREADS    RDB                           RETURN with XML listing of all threads
-    103      THREAD_CREATE   PYDB      -                   XML with thread information
-    104      THREAD_KILL     RDB       id                  kills the thread
-                             PYDB      id                  nofies RDB that thread was killed
-    105      THREAD_SUSPEND  RDB       XML of the stack,   suspends the thread
-                                       reason for suspension
-                             PYDB      id                  notifies RDB that thread was suspended
-    106      THREAD_RUN      RDB       id                  resume the thread
-                             PYDB      id \t reason        notifies RDB that thread was resumed
-    107      STEP_INTO       RDB       thread_id
-    108      STEP_OVER       RDB       thread_id
-    109      STEP_RETURN     RDB       thread_id
-    110      GET_VARIABLE    RDB       var_locator         GET_VARIABLE with XML of var content
-                                       see code for definition
-    111      SET_BREAK       RDB       file/line of the breakpoint
-    112      REMOVE_BREAK    RDB       file/line of the return
-    113      EVALUATE_EXPRESSION RDB   expression          result of evaluating the expression
-
+    101      RUN                  RDB       -                             -
+    102      LIST_THREADS         RDB                                     RETURN with XML listing of all threads
+    103      THREAD_CREATE        PYDB      -                             XML with thread information
+    104      THREAD_KILL          RDB       id                            kills the thread
+                                  PYDB      id                            nofies RDB that thread was killed
+    105      THREAD_SUSPEND       RDB       XML of the stack,             suspends the thread
+                                            reason for suspension
+                                  PYDB      id                            notifies RDB that thread was suspended
+    106      THREAD_RUN           RDB       id                            resume the thread
+                                  PYDB      id \t reason                  notifies RDB that thread was resumed
+    107      STEP_INTO            RDB       thread_id
+    108      STEP_OVER            RDB       thread_id
+    109      STEP_RETURN          RDB       thread_id
+    110      GET_VARIABLE         RDB       var_locator                   GET_VARIABLE with XML of var content
+                                            see code for definition
+    111      SET_BREAK            RDB       file/line of the breakpoint
+    112      REMOVE_BREAK         RDB       file/line of the return
+    113      EVALUATE_EXPRESSION  RDB       expression                    result of evaluating the expression
+    114      CMD_GET_FRAME        RDB
+    115      CMD_EXEC_EXPRESSION  RDB
+    116      CMD_WRITE_TO_CONSOLE PYDB
+    
 500 series diagnostics/ok
-    901      VERSION         either      Version string (1.0)               Currently just used at startup
-    902      RETURN          either      Depends on caller    -
+    901      VERSION              either      Version string (1.0)        Currently just used at startup
+    902      RETURN               either      Depends on caller    -
+    
 900 series: errors
-    501      ERROR           either      -                 This is reserved for unexpected errors.
+    501      ERROR                either      -                           This is reserved for unexpected errors.
                                   
     * RDB - remote debugger, the java end
     * PYDB - pydevd, the python end
-"""
+'''
 import traceback
-import StringIO
+
+try:
+    import cStringIO as StringIO #may not alway be available
+except:
+    import StringIO #@Reimport
 
 try:
     __setFalse = False
@@ -61,9 +69,7 @@ import sys
 import threading
 import Queue as PydevQueue
 from socket import socket
-from socket import AF_INET
-from socket import SOCK_STREAM
-from socket import error
+from socket import AF_INET, SOCK_STREAM, error #@UnresolvedImport
 import urllib
 import string
 import pydevd_vars
@@ -84,10 +90,14 @@ CMD_REMOVE_BREAK = 112
 CMD_EVALUATE_EXPRESSION = 113
 CMD_GET_FRAME = 114
 CMD_EXEC_EXPRESSION = 115
+CMD_WRITE_TO_CONSOLE = 116
 CMD_VERSION = 501
 CMD_RETURN = 502
 CMD_ERROR = 901 
 
+MAX_IO_MSG_SIZE = 400 #if the io is too big, we'll not send all (could make the debugger too non-responsive)
+                      #this number can be changed if there's need to do so
+
 VERSION_STRING = "1.0"
 
 
@@ -95,6 +105,8 @@ VERSION_STRING = "1.0"
 
 pydevd_trace = -1
 
+
+
 def pydevd_log(level, s):
     """ levels are: 
         0 most serious warnings/errors
@@ -107,7 +119,7 @@ def pydevd_log(level, s):
 
 def NormFile(filename):
     try:
-        rPath = os.path.realpath
+        rPath = os.path.realpath #@UndefinedVariable
     except:
         # jython does not support os.path.realpath
         # realpath is a no-op on systems without islink support
@@ -186,7 +198,13 @@ class WriterThread(PyDBDaemonThread):
         sys.settrace(None) # no debugging on this thread
         try:
             while not self.killReceived:
-                cmd = self.cmdQueue.get(1)
+                try:
+                    cmd = self.cmdQueue.get(1)
+                except:
+                    if pydevd_trace >= 0: print 'Finishing debug communication...(1)'
+                    #when liberating the thread here, we could have errors because we were shutting down
+                    #but the thread was still not liberated
+                    return
                 out = cmd.getOutgoing()
                 pydevd_log(1, "sending cmd " + out)
                 bytesSent = self.sock.send(out) #TODO: this does not guarantee that all message are sent (and jython does not have a send all)
@@ -197,6 +215,29 @@ class WriterThread(PyDBDaemonThread):
     
 
 
+#--------------------------------------------------- CREATING THE SOCKET THREADS
+    
+def startServer(port):
+    """ binds to a port, waits for the debugger to connect """
+    s = socket(AF_INET, SOCK_STREAM)
+    s.bind(('', port))
+    s.listen(1)
+    newSock, addr = s.accept()
+    return newSock
+
+def startClient(host, port):
+    """ connects to a host/port """
+    pydevd_log(1, "Connecting to " + host + ":" + str(port))
+    try:
+        s = socket(AF_INET, SOCK_STREAM);
+
+        s.connect((host, port))
+        pydevd_log(1, "Connected.")
+        return s
+    except:
+        print "server timed out after 10 seconds, could not connect to " + host + ":" + str(port)
+        print "Exiting. Bye!"
+        sys.exit(1)
 
 
     
@@ -246,7 +287,8 @@ class NetCommandFactory:
 
     def makeErrorMessage(self, seq, text):
         cmd = NetCommand(CMD_ERROR, seq, text)
-        #print >>sys.stderr, "Error: ", text
+        if pydevd_trace > 2:
+            print >>sys.stderr, "Error: ", text
         return cmd;
 
     def makeThreadCreatedMessage(self,thread):
@@ -259,12 +301,32 @@ class NetCommandFactory:
             t = threading.enumerate()
             cmdText = "<xml>"
             for i in t:
+#                print 'makeListThreadsMessage', t.additionalInfo.pydev_last_event
                 cmdText += self.threadToXML(i)
             cmdText += "</xml>"
             return NetCommand(CMD_RETURN, seq, cmdText)
         except:
             return self.makeErrorMessage(seq, sys.exc_info()[0])
 
+    def makeIoMessage(self, v, ctx, dbg=None):
+        '''
+        @param v: the message to pass to the debug server
+        @param ctx: 1 for stdio 2 for stderr
+        @param dbg: If not none, add to the writer
+        '''
+        
+        try:
+            if len(v) >  MAX_IO_MSG_SIZE:
+                v = v[0:MAX_IO_MSG_SIZE]
+                v += '...'
+                
+            v = pydevd_vars.makeValidXmlValue(urllib.quote(v, '/>_= \t'))
+            net = NetCommand(str(CMD_WRITE_TO_CONSOLE), 0, '<xml><io s="%s" ctx="%s"/></xml>' % (v, ctx))
+            if dbg:
+                dbg.writer.addCommand(net)
+        except:
+            return self.makeErrorMessage(0, sys.exc_info()[0])
+    
     def makeVersionMessage(self, seq):
         try:
             return NetCommand(CMD_VERSION, seq, VERSION_STRING)
@@ -296,12 +358,12 @@ class NetCommandFactory:
                 myId = str(id(curFrame))
                 #print "id is ", myId
                 
-                myName = curFrame.f_code.co_name
+                myName = curFrame.f_code.co_name #method name (if in method) or ? if global
                 #print "name is ", myName
                 
                 myFile = NormFile( curFrame.f_code.co_filename )                
-                #myFile = inspect.getsourcefile(curFrame) or inspect.getfile(frame)
                 #print "file is ", myFile
+                #myFile = inspect.getsourcefile(curFrame) or inspect.getfile(frame)
                 
                 myLine = str(curFrame.f_lineno)
                 #print "line is ", myLine
@@ -422,17 +484,20 @@ class InternalGetFrame(InternalThreadCommand):
     def doIt(self, dbg):
         """ Converts request into python variable """
         try:
-            xml = "<xml>"            
-            frame = pydevd_vars.findFrame(self.thread_id, self.frame_id)
-            xml += pydevd_vars.frameVarsToXML(frame)
-            xml += "</xml>"
-            cmd = dbg.cmdFactory.makeGetFrameMessage(self.sequence, xml)
-            dbg.writer.addCommand(cmd)
-        except Exception:
-            exc_info = sys.exc_info()
-            s = StringIO.StringIO()
-            traceback.print_exception(exc_info[0], exc_info[1], exc_info[2], file = s)
-            cmd = dbg.cmdFactory.makeErrorMessage(self.sequence, "Error resolving frame" + s.getvalue())
+            try:
+                xml = "<xml>"            
+                frame = pydevd_vars.findFrame(self.thread_id, self.frame_id)
+                xml += pydevd_vars.frameVarsToXML(frame)
+                xml += "</xml>"
+                cmd = dbg.cmdFactory.makeGetFrameMessage(self.sequence, xml)
+                dbg.writer.addCommand(cmd)
+            except pydevd_vars.FrameNotFoundError:
+                #don't print this error: frame not found: means that the client is dis-synchronized (but that's ok)
+                cmd = dbg.cmdFactory.makeErrorMessage(self.sequence, "Error resolving frame: %s from thread: %s" % (self.frame_id, self.thread_id))
+                dbg.writer.addCommand(cmd)
+        except:
+            traceback.print_exc()
+            cmd = dbg.cmdFactory.makeErrorMessage(self.sequence, "Error resolving frame: %s from thread: %s" % (self.frame_id, self.thread_id))
             dbg.writer.addCommand(cmd)
 
            
