@@ -25,6 +25,9 @@ import org.eclipse.core.variables.VariablesPlugin;
 import org.eclipse.debug.core.DebugPlugin;
 import org.eclipse.debug.core.ILaunchConfiguration;
 import org.eclipse.debug.core.ILaunchManager;
+import org.eclipse.jdt.core.IJavaProject;
+import org.eclipse.jdt.core.JavaModelException;
+import org.eclipse.jdt.launching.JavaRuntime;
 import org.python.copiedfromeclipsesrc.JavaVmLocationFinder;
 import org.python.pydev.core.REF;
 import org.python.pydev.debug.codecoverage.PyCoverage;
@@ -49,8 +52,8 @@ public class PythonRunnerConfig {
     public static final String RUN_REGULAR  = "RUN_REGULAR";
     public static final String RUN_UNITTEST = "RUN_UNITTEST";
     public static final String RUN_JYTHON   = "RUN_JYTHON";
-    
-    
+        
+    public IProject project;
 	public IPath resource;
 	public IPath interpreter;
 	public String[] arguments;
@@ -63,10 +66,6 @@ public class PythonRunnerConfig {
 	public int acceptTimeout = 5000; // miliseconds
 	public String[] envp = null;
 
-	// unit test specific
-	private int unitTestPort = 0;  // use getUnitTestPort
-	private String unitTestModule;
-	private String unitTestModuleDir;
     private String run;
     private ILaunchConfiguration configuration;
 
@@ -226,10 +225,6 @@ public class PythonRunnerConfig {
 		workingDirectory = workingPath == null ? null : workingPath.toFile();
 		acceptTimeout = PydevPrefs.getPreferences().getInt(PydevPrefs.CONNECT_TIMEOUT);
 
-        if (isUnittest()){
-			setUnitTestInfo();
-		}
-
 		//find the project
         IWorkspace w = ResourcesPlugin.getWorkspace();
         String projName = conf.getAttribute(Constants.ATTR_PROJECT, "");
@@ -237,7 +232,7 @@ public class PythonRunnerConfig {
             throw new CoreException(PydevDebugPlugin.makeStatus(IStatus.ERROR, "Unable to get project for the run",null));
         }
         
-        IProject project = w.getRoot().getProject(projName);
+        project = w.getRoot().getProject(projName);   
         
 
         if(project == null){ //Ok, we could not find it out
@@ -288,6 +283,7 @@ public class PythonRunnerConfig {
 	            String[] s = new String[envp.length+1];
 	            System.arraycopy(envp, 0, s, 0, envp.length);
 	            s[s.length-1] = "PYTHONPATH="+pythonpath;
+	            envp = s;
 		            
     		}
         }
@@ -332,28 +328,7 @@ public class PythonRunnerConfig {
 		return debugPort;		
 	}
 
-    public int getUnitTestPort(){
-		return unitTestPort;		
-	}
 
-    public void setUnitTestPort() throws CoreException {
-		unitTestPort = SocketUtil.findUnusedLocalPort("", 5000, 15000); //$NON-NLS-1$
-		if (unitTestPort == -1)
-			throw new CoreException(PydevDebugPlugin.makeStatus(IStatus.ERROR, "Could not find a free socket for unit test run", null));
-	}
-
-    private void setUnitTestInfo() throws CoreException {
-		setUnitTestPort();
-
-    	// get the test module name and path so that we can import it in Python
-		int segmentCount = resource.segmentCount();
-
-		IPath noextPath = resource.removeFileExtension();
-		unitTestModule =  noextPath.lastSegment();
-		IPath modulePath = resource.uptoSegment(segmentCount-1);
-		unitTestModuleDir = modulePath.toString();
-    }
-    
 	public String getRunningName() {
 		return resource.lastSegment();
 	}
@@ -393,7 +368,24 @@ public class PythonRunnerConfig {
     private String getRunFilesScript() throws CoreException {
         return REF.getFileAbsolutePath(PydevDebugPlugin.getScriptWithinPySrc("runfiles.py"));
     }
-
+    
+    
+	private String getClasspath(IJavaProject javaProject) throws JavaModelException {
+		
+		String[] cp = null;
+		try {
+			cp = JavaRuntime.computeDefaultRuntimeClassPath(javaProject);
+		} catch (CoreException e) {
+			e.printStackTrace();
+		}
+		String ret = "";
+		for (String s : cp){
+			ret += s + SimpleRunner.getPythonPathSeparator();
+		}
+		
+		return ret;
+		
+	}
 	/**
 	 * Create a command line for launching.
 	 * @return command line ready to be exec'd
@@ -412,9 +404,21 @@ public class PythonRunnerConfig {
 
             //some nice things on the classpath config: http://mindprod.com/jgloss/classpath.html
             cmdArgs.add("-classpath");
-            cmdArgs.add(interpreter+SimpleRunner.getPythonPathSeparator()+pythonpathUsed);
+            String cpath;
+            
+            //TODO: add some option in the project so that the user can choose to use the
+            //classpath specified in the java project instead of the pythonpath itself
+            
+//            if (project.getNature(Constants.JAVA_NATURE) != null){
+//            	cpath  = getClasspath(JavaCore.create(project));
+//            } else {
+            	cpath = interpreter + SimpleRunner.getPythonPathSeparator() + pythonpathUsed;
+//            }
+            cmdArgs.add(cpath);
             cmdArgs.add("-Dpython.path="+pythonpathUsed); //will be added to the env variables in the run (check if this works on all platforms...)
             
+           	addVmArgs(cmdArgs);
+            	
             if (isDebug) {
             	cmdArgs.add("-Dpython.security.respectJavaAccessibility=false"); //TODO: the user should configure this -- we use it so that 
             																	 //we can access the variables during debugging. 
@@ -445,6 +449,8 @@ public class PythonRunnerConfig {
                 cmdArgs.add("-u");
             }
         
+            addVmArgs(cmdArgs);
+            
     		if (isDebug) {
     			cmdArgs.add(getDebugScript());
                 cmdArgs.add("--type");
@@ -468,18 +474,7 @@ public class PythonRunnerConfig {
     		}
     
     		if(isUnittest()){
-                if (isFile()){
-        			cmdArgs.add(getUnitTestScript());
-        			cmdArgs.add(Integer.toString(getUnitTestPort()));
-        			cmdArgs.add(unitTestModuleDir);
-        			cmdArgs.add(unitTestModule);
-        			String[] retVal = new String[cmdArgs.size()];
-        			cmdArgs.toArray(retVal);
-        			return retVal;
-    
-                }else{ //run all testcases
-                    cmdArgs.add(getRunFilesScript());
-                }
+                cmdArgs.add(getRunFilesScript());
     		}
         }
         
@@ -496,8 +491,30 @@ public class PythonRunnerConfig {
 		return retVal;
 	}
 
+    /**
+     * @param cmdArgs
+     * @throws CoreException
+     */
+    private void addVmArgs(List<String> cmdArgs) throws CoreException {
+        String[] vmArguments = getVMArguments(configuration);
+        if(vmArguments != null){
+            for (int i = 0; i < vmArguments.length; i++){
+            	cmdArgs.add(vmArguments[i]);
+            }
+        }
+    }
+
 	
 	
+    private String[] getVMArguments(ILaunchConfiguration configuration) throws CoreException {
+    	String args = configuration.getAttribute(Constants.ATTR_VM_ARGUMENTS, (String) null);
+        if (args != null) {
+            String expanded = getStringVariableManager().performStringSubstitution(args);
+            return parseStringIntoList(expanded);
+       }
+       return null;
+	}
+
 	public String getCommandLineAsString() {
 		String[] args;
         try {
