@@ -22,20 +22,23 @@ import org.eclipse.jface.text.IDocument;
 import org.eclipse.jface.text.IDocumentListener;
 import org.eclipse.ui.IEditorInput;
 import org.eclipse.ui.IFileEditorInput;
-import org.python.parser.IParserHost;
-import org.python.parser.ParseException;
-import org.python.parser.PythonGrammar;
-import org.python.parser.ReaderCharStream;
-import org.python.parser.SimpleNode;
-import org.python.parser.TokenMgrError;
-import org.python.parser.ast.Module;
-import org.python.parser.ast.commentType;
 import org.python.pydev.core.ExtensionHelper;
 import org.python.pydev.core.IPyEdit;
 import org.python.pydev.core.IPythonNature;
+import org.python.pydev.core.Tuple;
 import org.python.pydev.core.docutils.DocUtils;
 import org.python.pydev.core.docutils.PySelection;
 import org.python.pydev.core.log.Log;
+import org.python.pydev.parser.jython.CharStream;
+import org.python.pydev.parser.jython.FastCharStream;
+import org.python.pydev.parser.jython.IParserHost;
+import org.python.pydev.parser.jython.ParseException;
+import org.python.pydev.parser.jython.PythonGrammar;
+import org.python.pydev.parser.jython.ReaderCharStream;
+import org.python.pydev.parser.jython.SimpleNode;
+import org.python.pydev.parser.jython.TokenMgrError;
+import org.python.pydev.parser.jython.ast.Module;
+import org.python.pydev.parser.jython.ast.commentType;
 
 /**
  * PyParser uses org.python.parser to parse the document (lexical analysis) It
@@ -53,6 +56,11 @@ public class PyParser {
      */
     public static boolean ACCEPT_NULL_EDITOR = false;
     
+    /**
+     * Defines whether we should use the fast stream or not
+     */
+    public static boolean USE_FAST_STREAM = true;
+    
     /**
      * To know whether we should try to do some reparse changing the input
      */
@@ -165,8 +173,8 @@ public class PyParser {
         parseNow(true);
     }
     
-    public void parseNow(boolean force){
-        scheduler.parseNow(force);
+    public void parseNow(boolean force, Object ... argsToReparse){
+        scheduler.parseNow(force, argsToReparse);
     }
 
     public void setDocument(IDocument document) {
@@ -219,16 +227,25 @@ public class PyParser {
      * stock listener implementation event is fired whenever we get a new root
      * @param original 
      */
-    protected void fireParserChanged(SimpleNode root, IAdaptable file, IDocument doc) {
+    @SuppressWarnings("unchecked")
+	protected void fireParserChanged(SimpleNode root, IAdaptable file, IDocument doc, Object ... argsToReparse) {
         this.root = root;
         synchronized(parserListeners){
-        	for (IParserObserver l : parserListeners) {
-        		l.parserChanged(root, file, doc);
+        	for (IParserObserver l : new ArrayList<IParserObserver>(parserListeners)) { //work on a copy (because listeners may want to remove themselves and we cannot afford concurrent modifications here)
+        		if(l instanceof IParserObserver2){
+        			((IParserObserver2)l).parserChanged(root, file, doc, argsToReparse);
+        		}else{
+        			l.parserChanged(root, file, doc);
+        		}
 			}
 
         	List<IParserObserver> participants = ExtensionHelper.getParticipants(ExtensionHelper.PYDEV_PARSER_OBSERVER);
             for (IParserObserver observer : participants) {
-                observer.parserChanged(root, file, doc);
+        		if(observer instanceof IParserObserver2){
+        			((IParserObserver2)observer).parserChanged(root, file, doc, argsToReparse);
+        		}else{
+        			observer.parserChanged(root, file, doc);
+        		}
             }
         }
     }
@@ -237,36 +254,43 @@ public class PyParser {
      * stock listener implementation event is fired when parse fails
      * @param original 
      */
-    protected void fireParserError(Throwable error, IAdaptable file, IDocument doc) {
+    @SuppressWarnings("unchecked")
+	protected void fireParserError(Throwable error, IAdaptable file, IDocument doc, Object ... argsToReparse) {
         synchronized(parserListeners){
-        	for (IParserObserver l : parserListeners) {
-                l.parserError(error, file, doc);
+        	for (IParserObserver l : new ArrayList<IParserObserver>(parserListeners)) {//work on a copy (because listeners may want to remove themselves and we cannot afford concurrent modifications here)
+        		if(l instanceof IParserObserver2){
+        			((IParserObserver2)l).parserError(error, file, doc, argsToReparse);
+        		}else{
+        			l.parserError(error, file, doc);
+        		}
             }
             List<IParserObserver> participants = ExtensionHelper.getParticipants(ExtensionHelper.PYDEV_PARSER_OBSERVER);
             for (IParserObserver observer : participants) {
-                observer.parserError(error, file, doc);
+            	if(observer instanceof IParserObserver2){
+            		((IParserObserver2)observer).parserError(error, file, doc, argsToReparse);
+            	}else{
+            		observer.parserError(error, file, doc);
+            	}
             }
         }
     }
 
     // ---------------------------------------------------------------------------- parsing
-    /**
-     * reparses the document getting the nature associated to the corresponding editor 
-     * @return
-     */
-    public Object[] reparseDocument() {
-    	return reparseDocument(editorView.getPythonNature());
-    }
+
     /**
      * Parses the document, generates error annotations
      * 
+     * @param argsToReparse: will be passed to fireParserError / fireParserChanged so that the IParserObserver2
+     * can check it. This is useful when the reparse was done with some specific thing in mind, so that its requestor
+     * can pass some specific thing to the parser observers
+     * 
      * @return a tuple with the SimpleNode root(if parsed) and the error (if any).
      *         if we are able to recover from a reparse, we have both, the root and the error.
      */
-    public Object[] reparseDocument(IPythonNature nature) {
+    public Tuple<SimpleNode, Throwable> reparseDocument(Object ... argsToReparse) {
         
         //get the document ast and error in object
-        Object obj[] = reparseDocument(new ParserInfo(document, true, nature, -1));
+        Tuple<SimpleNode, Throwable> obj = reparseDocument(new ParserInfo(document, true, -1));
         
         IFile original = null;
         IAdaptable adaptable = null;
@@ -288,30 +312,36 @@ public class PyParser {
             }
         }
         
-        if(obj[0] != null && obj[0] instanceof SimpleNode){
+        //delete the markers
+        if (original != null){
+        	try {
+        		IMarker[] markers = original.findMarkers(IMarker.PROBLEM, false, IResource.DEPTH_ZERO);
+        		if(markers.length > 0){
+        			original.deleteMarkers(IMarker.PROBLEM, false, IResource.DEPTH_ZERO);
+        		}
+        	} catch (CoreException e) {
+        		Log.log(e);
+        	}
+        	
+        }else if(adaptable == null){
+        	//ok, we have nothing... maybe we are in tests...
+        	if (!PyParser.ACCEPT_NULL_EDITOR){
+        		throw new RuntimeException("Null editor received in parser!");
+        	}
+        }
+        //end delete the markers
+        
+        if(obj.o1 != null){
             //ok, reparse succesful, lets erase the markers that are in the editor we just parsed
-            if (original != null){
-                try {
-                    original.deleteMarkers(IMarker.PROBLEM, false, IResource.DEPTH_ZERO);
-                } catch (CoreException e) {
-                    Log.log(e);
-                }
-                
-            }else if(adaptable == null){
-                //ok, we have nothing... maybe we are in tests...
-                if (!PyParser.ACCEPT_NULL_EDITOR){
-                    throw new RuntimeException("Null editor received in parser!");
-                }
-            }
-            fireParserChanged((SimpleNode) obj[0], adaptable, document);
+            fireParserChanged(obj.o1, adaptable, document, argsToReparse);
         }
         
-        if(obj[1] != null && obj[1] instanceof ParseException){
-            fireParserError((ParseException) obj[1], adaptable, document);
+        if(obj.o2 != null && obj.o2 instanceof ParseException){
+            fireParserError((ParseException) obj.o2, adaptable, document, argsToReparse);
         }
         
-        if(obj[1] != null && obj[1] instanceof TokenMgrError){
-            fireParserError((TokenMgrError) obj[1], adaptable, document);
+        if(obj.o2 != null && obj.o2 instanceof TokenMgrError){
+            fireParserError((TokenMgrError) obj.o2, adaptable, document, argsToReparse);
         }
         
         return obj;
@@ -325,19 +355,25 @@ public class PyParser {
     public static class ParserInfo{
         public IDocument document;
         public boolean stillTryToChangeCurrentLine=true; 
-        public IPythonNature nature;
         public int currentLine=-1;
         public String initial = null;
-        public List linesChanged = new ArrayList();
+        public List<Integer> linesChanged = new ArrayList<Integer>();
         public ParseException parseErr;
         public boolean tryReparse = TRY_REPARSE;
         
+        public ParserInfo(IDocument document, boolean changedCurrentLine){
+            this(document, changedCurrentLine, null);
+        }
+        
         public ParserInfo(IDocument document, boolean changedCurrentLine, IPythonNature nature){
             this.document = document;
             this.stillTryToChangeCurrentLine = changedCurrentLine;
-            this.nature = nature;
         }
 
+        public ParserInfo(IDocument document, boolean changedCurrentLine, int currentLine){
+            this(document, changedCurrentLine, null, currentLine);
+        }
+        
         public ParserInfo(IDocument document, boolean changedCurrentLine, IPythonNature nature, int currentLine){
             this(document, changedCurrentLine, nature);
             this.currentLine = currentLine;
@@ -348,7 +384,7 @@ public class PyParser {
      * @return a tuple with the SimpleNode root(if parsed) and the error (if any).
      *         if we are able to recover from a reparse, we have both, the root and the error.
      */
-    public static Object[] reparseDocument(ParserInfo info) {
+    public static Tuple<SimpleNode, Throwable> reparseDocument(ParserInfo info) {
         // create a stream with document's data
         String startDoc = info.document.get();
         if(info.initial == null){
@@ -357,11 +393,25 @@ public class PyParser {
 
         IDocument newDoc = new Document(startDoc);
         StringBuffer endingComments = PySelection.removeEndingComments(newDoc);
+        try {
+            //make sure it ends with a new line
+            newDoc.replace(newDoc.getLength(), 0, "\n");
+        } catch (Exception e) {
+            throw new RuntimeException(e);
+        }
         String initialDoc = newDoc.get();
         
         
-        StringReader inString = new StringReader(initialDoc);
-        ReaderCharStream in = new ReaderCharStream(inString);
+        CharStream in = null;
+        if(USE_FAST_STREAM){
+        	in = new FastCharStream(initialDoc);
+        }else{
+        	//this should be deprecated in the future (it is still here so that we can evaluate
+        	//the changes done by the change of the reader).
+	        StringReader inString = new StringReader(initialDoc);
+	        in = new ReaderCharStream(inString);
+        }
+        
         IParserHost host = new CompilerAPI();
         PythonGrammar grammar = null;
 
@@ -377,7 +427,7 @@ public class PyParser {
                 Module m = (Module) newRoot;
                 m.addSpecial(new commentType(endingComments.toString()), true);
             }
-            return new Object[]{newRoot,null};
+            return new Tuple<SimpleNode, Throwable>(newRoot,null);
 		
 
         } catch (ParseException parseErr) {
@@ -396,7 +446,7 @@ public class PyParser {
                     newRoot = tryReparseAgain(info, info.parseErr);
                 }
             }
-            return new Object[]{newRoot, parseErr};
+            return new Tuple<SimpleNode, Throwable>(newRoot, parseErr);
             
         
         } catch (TokenMgrError tokenErr) {
@@ -408,11 +458,11 @@ public class PyParser {
                 }
             }
             
-            return new Object[]{newRoot, tokenErr};
+            return new Tuple<SimpleNode, Throwable>(newRoot, tokenErr);
 
         } catch (Exception e) {
             Log.log(e);
-            return new Object[]{null, null};
+            return new Tuple<SimpleNode, Throwable>(null, null);
         
         } catch (Throwable e) {
 			//PythonGrammar$LookaheadSuccess error: this happens sometimes when the file is
@@ -422,7 +472,7 @@ public class PyParser {
 			}else{
 				Log.log(e);
 			}
-			return new Object[]{null, null};
+			return new Tuple<SimpleNode, Throwable>(null, null);
         }
     }
 
@@ -513,7 +563,7 @@ public class PyParser {
             Document doc = new Document(docToParse);
             info.document = doc;
             info.stillTryToChangeCurrentLine = false;
-	        return (SimpleNode) reparseDocument(info)[0];
+	        return reparseDocument(info).o1;
         }
         return null;
     }
