@@ -3,36 +3,43 @@
  */
 package org.python.pydev.parser.visitors;
 
-import java.lang.reflect.Field;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.Iterator;
 import java.util.List;
 
-import org.python.parser.SimpleNode;
-import org.python.parser.ast.Attribute;
-import org.python.parser.ast.Call;
-import org.python.parser.ast.ClassDef;
-import org.python.parser.ast.Expr;
-import org.python.parser.ast.FunctionDef;
-import org.python.parser.ast.Import;
-import org.python.parser.ast.ImportFrom;
-import org.python.parser.ast.ListComp;
-import org.python.parser.ast.NameTok;
-import org.python.parser.ast.Num;
-import org.python.parser.ast.Str;
-import org.python.parser.ast.Subscript;
-import org.python.parser.ast.Tuple;
-import org.python.parser.ast.aliasType;
-import org.python.parser.ast.exprType;
-import org.python.parser.ast.stmtType;
+import org.python.pydev.core.FullRepIterable;
 import org.python.pydev.core.REF;
-import org.python.pydev.core.log.Log;
+import org.python.pydev.parser.jython.SimpleNode;
+import org.python.pydev.parser.jython.ast.Attribute;
+import org.python.pydev.parser.jython.ast.Call;
+import org.python.pydev.parser.jython.ast.ClassDef;
+import org.python.pydev.parser.jython.ast.Dict;
+import org.python.pydev.parser.jython.ast.Expr;
+import org.python.pydev.parser.jython.ast.FunctionDef;
+import org.python.pydev.parser.jython.ast.Import;
+import org.python.pydev.parser.jython.ast.ImportFrom;
+import org.python.pydev.parser.jython.ast.ListComp;
+import org.python.pydev.parser.jython.ast.Name;
+import org.python.pydev.parser.jython.ast.NameTok;
+import org.python.pydev.parser.jython.ast.Num;
+import org.python.pydev.parser.jython.ast.Str;
+import org.python.pydev.parser.jython.ast.Subscript;
+import org.python.pydev.parser.jython.ast.Tuple;
+import org.python.pydev.parser.jython.ast.aliasType;
+import org.python.pydev.parser.jython.ast.commentType;
+import org.python.pydev.parser.jython.ast.excepthandlerType;
+import org.python.pydev.parser.jython.ast.exprType;
+import org.python.pydev.parser.jython.ast.keywordType;
+import org.python.pydev.parser.jython.ast.stmtType;
+import org.python.pydev.parser.visitors.scope.ASTEntry;
+import org.python.pydev.parser.visitors.scope.EasyASTIteratorVisitor;
 
 public class NodeUtils {
 
     /**
-     * @param node
-     * @return
+     * @param node a function definition (if other will return an empty string)
+     * @return a string with the representation of the parameters of the function
      */
     public static String getNodeArgs(SimpleNode node) {
         if(node instanceof FunctionDef){
@@ -80,43 +87,65 @@ public class NodeUtils {
      * @return A suitable String representation for some node.
      */
     public static String getRepresentationString(SimpleNode node, boolean useTypeRepr) {
+        if(node instanceof NameTok){
+            NameTok tok = (NameTok) node;
+            return tok.id;
+        }
+        
+        if(node instanceof Name){
+            Name name = (Name) node;
+            return name.id;
+        }
+        
+        if(node instanceof aliasType){
+            aliasType type = (aliasType) node;
+            return ((NameTok)type.name).id;
+        }
+        if(node instanceof Attribute){
+            Attribute attribute = (Attribute) node;
+            return discoverRep(attribute.attr);
             
-        if (REF.hasAttr(node, "name")) {
-            Object attrObj = REF.getAttrObj(node, "name");
-            if(attrObj instanceof NameTok){
-                NameTok n = (NameTok) attrObj;
-                return n.id;
-            }
-            return attrObj.toString();
-            
-        }else if (REF.hasAttr(node, "id")) {
-            return discoverRep(REF.getAttrObj(node, "id"));
-    
-        }else if (REF.hasAttr(node, "attr")) {
-            return discoverRep(REF.getAttrObj(node, "attr"));
-    
-        }else if (REF.hasAttr(node, "arg")) {
-            return discoverRep(REF.getAttrObj(node, "arg"));
-            
-        }else if (node instanceof Call){
+        }
+        
+        if(node instanceof keywordType){
+            keywordType type = (keywordType) node;
+            return discoverRep(type.arg);
+        }
+        
+        if(node instanceof ClassDef){
+            ClassDef def = (ClassDef) node;
+            return ((NameTok)def.name).id;
+        }
+        
+        if(node instanceof FunctionDef){
+            FunctionDef def = (FunctionDef) node;
+            return ((NameTok)def.name).id;
+        }
+        
+        if (node instanceof Call){
             Call call = ((Call)node);
             return getRepresentationString(call.func, useTypeRepr);
-            
-        }else if (node instanceof org.python.parser.ast.List || node instanceof ListComp){
+        }
+        
+        if (node instanceof org.python.pydev.parser.jython.ast.List || node instanceof ListComp){
             String val = "[]";
             if(useTypeRepr){
             	val = getBuiltinType(val);
             }
 			return val;
-    
-        }else if (node instanceof Str){
-            String val = "'"+((Str)node).s+"'";
+        }
+        
+        if (node instanceof Str){
+            String val;
             if(useTypeRepr){
-            	val = getBuiltinType(val);
+            	val = getBuiltinType("''");
+            }else{
+                val = "'"+((Str)node).s+"'";
             }
 			return val;
-
-        }else if (node instanceof Tuple){
+        }
+        
+        if (node instanceof Tuple){
             StringBuffer buf = new StringBuffer();
             Tuple t = (Tuple)node;
             for ( exprType e : t.elts){
@@ -133,16 +162,17 @@ public class NodeUtils {
             	val = getBuiltinType(val);
             }
 			return val;
-
-            
-        }else if (node instanceof Num){
+        }
+        
+        if (node instanceof Num){
             String val = ((Num)node).n.toString();
             if(useTypeRepr){
             	val = getBuiltinType(val);
             }
             return val;
-            
-        }else if (node instanceof Import){
+        }
+        
+        if (node instanceof Import){
             aliasType[] names = ((Import)node).names;
             for (aliasType n : names) {
                 if(n.asname != null){
@@ -152,6 +182,19 @@ public class NodeUtils {
             }
         }
         
+
+        if(node instanceof commentType){
+            commentType type = (commentType) node;
+            return type.id;
+        }
+        
+        if(node instanceof excepthandlerType){
+            excepthandlerType type = (excepthandlerType) node;
+            return type.name.toString();
+            
+        }
+
+        
         return null;
     }
 
@@ -160,26 +203,22 @@ public class NodeUtils {
      * @param t
      */
     public static String getNodeDocString(SimpleNode node) {
-        //and check if it has a docstring.
-        if (REF.hasAttr(node, "body")) {
-    
-            Field field = REF.getAttr(node, "body");
-            try {
-                Object obj = field.get(node);
-                if (obj instanceof stmtType[]) {
-                    stmtType body[] = (stmtType[]) obj;
-                    if (body.length > 0) {
-                        if (body[0] instanceof Expr) {
-                            Expr e = (Expr) body[0];
-                            if (e.value instanceof Str) {
-                                Str s = (Str) e.value;
-                                return s.s;
-                            }
-                        }
-                    }
+        stmtType body[] = null;
+        if(node instanceof FunctionDef){
+            FunctionDef def = (FunctionDef) node;
+            body = def.body;
+        } else if(node instanceof ClassDef){
+            ClassDef def = (ClassDef) node;
+            body = def.body;
+            
+        }
+        if (body != null && body.length > 0) {
+            if (body[0] instanceof Expr) {
+                Expr e = (Expr) body[0];
+                if (e.value instanceof Str) {
+                    Str s = (Str) e.value;
+                    return s.s;
                 }
-            } catch (Exception e) {
-                Log.log(e);
             }
         }
         return null;
@@ -188,8 +227,10 @@ public class NodeUtils {
     
     
     public static String getFullRepresentationString(SimpleNode node) {
-
-        
+    	if (node instanceof Dict){
+    		return "dict";
+    	}
+    	
         if (node instanceof Str || node instanceof Num){
             return getRepresentationString(node, true);
         } 
@@ -289,11 +330,17 @@ public class NodeUtils {
      * @return the line definition of a node
      */
     public static int getLineDefinition(SimpleNode ast2) {
-        if(ast2 instanceof Attribute){
-            if(!(((Attribute)ast2).value instanceof Call)){
-                return getLineDefinition(((Attribute)ast2).value);
+        while(ast2 instanceof Attribute){
+            exprType val = ((Attribute)ast2).value;
+            if(!(val instanceof Call)){
+                ast2 = val;
+            }else{
+                break;
             }
         }
+        if(ast2 instanceof FunctionDef){
+            return ((FunctionDef)ast2).name.beginLine;
+        }
         return ast2.beginLine;
     }
 
@@ -398,11 +445,10 @@ public class NodeUtils {
     public static int getLineEnd(SimpleNode v) {
         if(v instanceof Str){
             String s = ((Str)v).s;
-            char[] cs = s.toCharArray();
             int found = 0;
-            for (int i = 0; i < cs.length; i++) {
+            for (int i = 0; i < s.length(); i++) {
      
-                if(cs[i] == '\n'){
+                if(s.charAt(i) == '\n'){
                     found += 1;
                 }
             }
@@ -464,8 +510,8 @@ public class NodeUtils {
      * @return a list with the attribute parts in its forward order, and not backward as presented
      * in the grammar.
      */
-	public static List getAttributeParts(Attribute node) {
-		ArrayList<Object> nodes = new ArrayList<Object>();
+	public static List<SimpleNode> getAttributeParts(Attribute node) {
+		ArrayList<SimpleNode> nodes = new ArrayList<SimpleNode>();
 		
 		nodes.add(node.attr);
 		SimpleNode s = node.value;
@@ -490,6 +536,100 @@ public class NodeUtils {
 		return nodes;
 	}
 
+
+    /**
+     * Gets the parent names for a class definition
+     * 
+     * @param onlyLastSegment determines whether we should return only the last segment if the name
+     * of the parent resolves to a dotted name.
+     */
+    public static List<String> getParentNames(ClassDef def, boolean onlyLastSegment) {
+        ArrayList<String> ret = new ArrayList<String>();
+        for(exprType base: def.bases){
+            String rep = getFullRepresentationString(base);
+            if(onlyLastSegment){
+                rep = FullRepIterable.getLastPart(rep);
+            }
+            ret.add(rep);
+        }
+        return ret;
+    }
+
+
+    /**
+     * @return true if the node is an import node (and false otherwise).
+     */
+	public static boolean isImport(SimpleNode ast) {
+        if(ast instanceof Import || ast instanceof ImportFrom){
+            return true;
+        }
+        return false;
+	}
+
+
+	public static NameTok getNameForAlias(aliasType t) {
+		if(t.asname != null){
+			return (NameTok) t.asname;
+		}else{
+			return (NameTok) t.name;
+		}
+	}
+
+
+	public static NameTok getNameForRep(aliasType[] names, String representation) {
+		for (aliasType name : names) {
+			NameTok nameForAlias = getNameForAlias(name);
+			String aliasRep = NodeUtils.getRepresentationString(nameForAlias);
+			if(representation.equals(aliasRep)){
+				return nameForAlias;
+			}
+		}
+		return null;
+	}
+
+
+    /**
+     * @param lineNumber the line we want to get the context from
+     * @param ast the ast that corresponds to our context
+     * @return the full name for the context where we are (in the format Class.method.xxx.xxx)
+     */
+    public static String getContextName(int lineNumber, SimpleNode ast) {
+        if(ast != null){
+            EasyASTIteratorVisitor visitor = EasyASTIteratorVisitor.create(ast);
+            Iterator<ASTEntry> classesAndMethodsIterator = visitor.getClassesAndMethodsIterator();
+            ASTEntry last = null;
+            while (classesAndMethodsIterator.hasNext()) {
+                ASTEntry entry = classesAndMethodsIterator.next();
+                if(entry.node.beginLine > lineNumber+1){
+                    //ok, now, let's find out which context actually contains it...
+                    break;
+                }                
+                last = entry;
+            }
+            
+            while(last != null && last.endLine <= lineNumber){
+                last = last.parent;
+            }
+            
+            if(last != null){
+    
+                StringBuffer buffer = new StringBuffer();
+                boolean first = true;
+                while (last != null){
+                    String name = last.getName();
+                    buffer.insert(0, name);
+                    last = last.parent;
+                    if(!first){
+                        buffer.insert(name.length(),".");
+                    }
+                    first = false;
+                }
+                return buffer.toString();
+            }
+        }
+    	return null;
+    }
+
     
 
 }
