@@ -1,12 +1,8 @@
-/*
- * @author: atotic
- * Created: July 10, 2003
- * License: Common Public License v1.0
- */
 package org.python.pydev.editor;
 
 import java.io.File;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.HashMap;
 import java.util.List;
 import java.util.ListResourceBundle;
@@ -25,6 +21,7 @@ import org.eclipse.core.runtime.Preferences;
 import org.eclipse.core.runtime.Status;
 import org.eclipse.core.runtime.jobs.Job;
 import org.eclipse.jface.action.IAction;
+import org.eclipse.jface.action.IStatusLineManager;
 import org.eclipse.jface.preference.IPreferenceStore;
 import org.eclipse.jface.text.Assert;
 import org.eclipse.jface.text.BadLocationException;
@@ -33,6 +30,11 @@ import org.eclipse.jface.text.IRegion;
 import org.eclipse.jface.text.source.ISourceViewer;
 import org.eclipse.jface.text.source.SourceViewer;
 import org.eclipse.swt.SWT;
+import org.eclipse.swt.graphics.Font;
+import org.eclipse.swt.graphics.FontData;
+import org.eclipse.swt.widgets.Display;
+import org.eclipse.ui.IActionBars;
+import org.eclipse.ui.IEditorActionBarContributor;
 import org.eclipse.ui.IEditorInput;
 import org.eclipse.ui.IEditorSite;
 import org.eclipse.ui.IFileEditorInput;
@@ -40,36 +42,43 @@ import org.eclipse.ui.IStorageEditorInput;
 import org.eclipse.ui.PartInitException;
 import org.eclipse.ui.editors.text.ILocationProvider;
 import org.eclipse.ui.editors.text.TextFileDocumentProvider;
+import org.eclipse.ui.internal.util.SWTResourceUtil;
+import org.eclipse.ui.part.EditorActionBarContributor;
 import org.eclipse.ui.part.FileEditorInput;
 import org.eclipse.ui.texteditor.AbstractDecoratedTextEditorPreferenceConstants;
 import org.eclipse.ui.texteditor.DefaultRangeIndicator;
+import org.eclipse.ui.texteditor.IDocumentProvider;
 import org.eclipse.ui.texteditor.IEditorStatusLine;
 import org.eclipse.ui.texteditor.ITextEditorActionDefinitionIds;
 import org.eclipse.ui.texteditor.MarkerUtilities;
 import org.eclipse.ui.texteditor.TextOperationAction;
 import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
 import org.python.copiedfromeclipsesrc.PydevFileEditorInput;
-import org.python.parser.ParseException;
-import org.python.parser.SimpleNode;
-import org.python.parser.Token;
-import org.python.parser.TokenMgrError;
 import org.python.pydev.builder.PyDevBuilderPrefPage;
 import org.python.pydev.core.ExtensionHelper;
 import org.python.pydev.core.IPyEdit;
 import org.python.pydev.core.IPythonNature;
 import org.python.pydev.core.REF;
+import org.python.pydev.core.docutils.PyPartitionScanner;
+import org.python.pydev.core.docutils.PySelection;
+import org.python.pydev.editor.actions.OfflineAction;
+import org.python.pydev.editor.actions.OfflineActionTarget;
 import org.python.pydev.editor.actions.PyOpenAction;
+import org.python.pydev.editor.autoedit.IIndentPrefs;
 import org.python.pydev.editor.autoedit.PyAutoIndentStrategy;
-import org.python.pydev.editor.codecompletion.revisited.PythonPathHelper;
 import org.python.pydev.editor.codecompletion.shell.AbstractShell;
 import org.python.pydev.editor.codefolding.CodeFoldingSetter;
 import org.python.pydev.editor.codefolding.PyEditProjection;
-import org.python.pydev.editor.model.AbstractNode;
 import org.python.pydev.editor.model.IModelListener;
-import org.python.pydev.editor.model.Location;
-import org.python.pydev.editor.model.ModelMaker;
+import org.python.pydev.editor.scripting.PyEditScripting;
 import org.python.pydev.outline.PyOutlinePage;
 import org.python.pydev.parser.PyParser;
+import org.python.pydev.parser.jython.ParseException;
+import org.python.pydev.parser.jython.SimpleNode;
+import org.python.pydev.parser.jython.Token;
+import org.python.pydev.parser.jython.TokenMgrError;
+import org.python.pydev.parser.visitors.NodeUtils;
+import org.python.pydev.parser.visitors.scope.ASTEntry;
 import org.python.pydev.plugin.PydevPlugin;
 import org.python.pydev.plugin.PydevPrefs;
 import org.python.pydev.plugin.nature.PythonNature;
@@ -118,9 +127,6 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
     /** need to hold onto it to support indentPrefix change through preferences */
     PyEditConfiguration editConfiguration;
 
-    /** Python model */
-    AbstractNode pythonModel;
-
     /**
      * AST that created python model
      */
@@ -132,8 +138,104 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
     /** listeners that get notified of model changes */
     List<IModelListener> modelListeners;
 
+    // ---------------------------- listeners stuff
+    /**
+     * Those are the ones that register with the PYDEV_PYEDIT_LISTENER extension point
+     */
     private static List<IPyEditListener> editListeners;
     
+    /**
+     * Those are the ones that register at runtime (not throught extensions points).
+     */
+    private List<IPyEditListener> registeredEditListeners = new ArrayList<IPyEditListener>();
+
+    /**
+     * This is the scripting engine that is binded to this interpreter.
+     */
+	private PyEditScripting pyEditScripting;
+
+	/**
+	 * Lock for initialization sync
+	 */
+	private Object lock = new Object();
+    
+    public void addPyeditListener(IPyEditListener listener){
+        registeredEditListeners.add(listener);
+    }
+    
+    public void removePyeditListener(IPyEditListener listener){
+        registeredEditListeners.remove(listener);
+    }
+
+    
+
+    @Override
+    protected void handleCursorPositionChanged() {
+        super.handleCursorPositionChanged();
+        if(!initFinished){
+        	return;
+        }
+        for(IPyEditListener listener : getAllListeners()){
+            try {
+                if(listener instanceof IPyEditListener2){
+                    ((IPyEditListener2)listener).handleCursorPositionChanged(this);
+                }
+            } catch (Throwable e) {
+                //must not fail
+                PydevPlugin.log(e);
+            }
+        }
+    }
+
+    public List<IPyEditListener> getAllListeners() {
+    	while (initFinished == false){
+    		synchronized(getLock()){
+    			try {
+    				getLock().wait();
+				} catch (Exception e) {
+					//ignore
+					e.printStackTrace();
+				}
+    		}
+    	}
+        ArrayList<IPyEditListener> listeners = new ArrayList<IPyEditListener>();
+        if(editListeners != null){
+            listeners.addAll(editListeners);
+        }
+        listeners.addAll(registeredEditListeners);
+        return listeners;
+    }
+
+    private Object getLock() {
+		return lock;
+	}
+
+	/**
+     * This map may be used by clients to store info regarding this editor.
+     * 
+     * Clients should be careful so that this key is unique and does not conflict with other
+     * plugins. 
+     * 
+     * This is not enforced.
+     * 
+     * The suggestion is that the cache key is always preceded by the class name that will use it.
+     */
+    public Map<String,Object> cache = new HashMap<String, Object>();
+
+    /**
+     * Indicates whether the init was already finished
+     */
+	protected boolean initFinished = false;
+
+	private PyEditNotifier notifier;
+
+	private boolean disposed = false;
+	public boolean isDisposed() {
+		return disposed;
+	}
+    
+    // ---------------------------- end listeners stuff
+    
     @SuppressWarnings("unchecked")
 	public PyEdit() {
         super();
@@ -160,6 +262,7 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
 
     }
 
+    
     /**
      * Sets the forceTabs preference for auto-indentation.
      * 
@@ -170,10 +273,12 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
      */
     private void resetForceTabs() {
         IDocument doc = getDocumentProvider().getDocument(getEditorInput());
-        if (doc == null)
+        if (doc == null){
             return;
+        }
+        
         if (!PydevPrefs.getPreferences().getBoolean(PydevPrefs.GUESS_TAB_SUBSTITUTION)) {
-            indentStrategy.getIndentPrefs().setForceTabs(false);
+            getIndentPrefs().setForceTabs(false);
             return;
         }
 
@@ -199,7 +304,7 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
             }
             i++;
         }
-        indentStrategy.getIndentPrefs().setForceTabs(forceTabs);
+        getIndentPrefs().setForceTabs(forceTabs);
         editConfiguration.resetIndentPrefixes();
         // display a message in the status line
         if (forceTabs) {
@@ -209,14 +314,22 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
         }
     }
 
+    /**
+     * @return the indentation preferences
+     */
+    public IIndentPrefs getIndentPrefs() {
+        return indentStrategy.getIndentPrefs();
+    }
+
     /**
      * Initializes everyone that needs document access
      *  
      */
     public void init(final IEditorSite site, final IEditorInput input) throws PartInitException {
+    	notifier = new PyEditNotifier(this);
         super.init(site, input);
 
-        IDocument document = getDocument(input);
+        final IDocument document = getDocument(input);
         
         // check the document partitioner (sanity check / fix)
         PyPartitionScanner.checkPartitionScanner(document);
@@ -231,7 +344,7 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
         
         //we also want to initialize our shells...
         //we use 2: one for refactoring and one for code completion.
-        new Thread() {
+        Thread thread2 = new Thread() {
             public void run() {
                 try {
                     //ok, behaviour change... we just initialize the shell that is not for code completion
@@ -248,12 +361,11 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
                 }
 
             }
-        }.start();
+        };
+        thread2.setName("Shell starter");
+        thread2.start();
 
         
-        //set the parser for the document
-        parser.setDocument(document);
-
         // listen to changes in TAB_WIDTH preference
         prefListener = new Preferences.IPropertyChangeListener() {
             public void propertyChange(Preferences.PropertyChangeEvent event) {
@@ -264,7 +376,11 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
                     if (sourceViewer == null){
                         return;
                     }
+                    getIndentPrefs().regenerateIndentString();
                     sourceViewer.getTextWidget().setTabs(PydevPlugin.getDefault().getPluginPreferences().getInt(PydevPrefs.TAB_WIDTH));
+                    
+                }else if (property.equals(PydevPrefs.SUBSTITUTE_TABS)) {
+                	getIndentPrefs().regenerateIndentString();
                    
                 //auto adjust for file tabs
                 } else if (property.equals(PydevPrefs.GUESS_TAB_SUBSTITUTION)) {
@@ -295,6 +411,29 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
         };
         resetForceTabs();
         PydevPrefs.getPreferences().addPropertyChangeListener(prefListener);
+        
+    
+        Runnable runnable = new Runnable(){
+
+			public void run() {
+				//let's do that in a thread, so that we don't have any delays in setting up the editor
+				pyEditScripting = new PyEditScripting();
+				addPyeditListener(pyEditScripting);
+				
+				//set the parser for the document
+				parser.setDocument(document);
+				notifier.notifyOnSetDocument(document);
+				initFinished = true;
+				synchronized(getLock()){
+					getLock().notifyAll();
+				}
+			}
+        };
+        Thread thread = new Thread(runnable);
+        thread.setPriority(Thread.MIN_PRIORITY);
+        thread.setName("PyEdit initializer");
+        thread.start();
+
     }
 
     /**
@@ -329,6 +468,7 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
             parser.setDocument(document);
             PyPartitionScanner.checkPartitionScanner(document);
         }
+        notifier.notifyOnSetDocument(document);
     }
     
     /**
@@ -340,11 +480,14 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
     }
 
     /**
-     * @param input
-     * @return
+     * @return the document that is binded to this editor (may be null)
      */
     public IDocument getDocument() {
-        return getDocumentProvider().getDocument(getEditorInput());
+        IDocumentProvider documentProvider = getDocumentProvider();
+        if(documentProvider != null){
+            return documentProvider.getDocument(getEditorInput());
+        }
+        return null;
     }
     
     /** 
@@ -354,18 +497,9 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
         fixEncoding(getEditorInput(), getDocument());
         super.performSave(overwrite, progressMonitor);
         parser.notifySaved();
-        if(editListeners != null){
-	        for(IPyEditListener listener : editListeners){
-	        	try {
-                    listener.onSave(this);
-                } catch (Exception e) {
-                    //must not fail
-                    PydevPlugin.log(e);
-                }
-	        }
-        }
+        notifier.notifyOnSave();
     }
-    
+
     /**
      * Forces the encoding to the one specified in the file
      * 
@@ -375,7 +509,7 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
     private void fixEncoding(final IEditorInput input, IDocument document) {
         if (input instanceof FileEditorInput) {
             final IFile file = (IFile) ((FileEditorInput) input).getAdapter(IFile.class);
-            final String encoding = PythonPathHelper.getPythonFileEncoding(document);
+            final String encoding = REF.getPythonFileEncoding(document, file.getFullPath().toOSString());
             if (encoding != null) {
                 try {
                     if (encoding.equals(file.getCharset()) == false) {
@@ -413,6 +547,16 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
     }
     
 
+    public IFile getIFile() {
+        IEditorInput editorInput = this.getEditorInput();
+        if (editorInput instanceof FileEditorInput) {
+            IFile file = (IFile) ((FileEditorInput) editorInput).getAdapter(IFile.class);
+            return file;
+        }
+        return null;
+        
+    }
+    
     /**
      * @return
      *  
@@ -443,12 +587,19 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
 
     // cleanup
     public void dispose() {
+    	this.disposed = true;
+    	notifier.notifyOnDispose();
+
         PydevPrefs.getPreferences().removePropertyChangeListener(prefListener);
         parser.dispose();
         colorCache.dispose();
+        pyEditScripting = null;
+        cache.clear();
+        cache = null;
         super.dispose();
     }
 
+
     private static final String TEMPLATE_PROPOSALS_ID = "org.python.pydev.editors.PyEdit.TemplateProposals";
 
     private static final String CONTENTASSIST_PROPOSAL_ID = "org.python.pydev.editors.PyEdit.ContentAssistProposal";
@@ -461,7 +612,7 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
 
     public static final int SIMPLEASSIST_PROPOSALS = 999778;
     
-    private static class MyResources extends ListResourceBundle {
+    public static class MyResources extends ListResourceBundle {
         public Object[][] getContents() {
             return contents;
         }
@@ -532,19 +683,19 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
         setAction(ACTION_OPEN, openAction);
         enableBrowserLikeLinks();
         
-        if(editListeners != null){
-            for(IPyEditListener listener : editListeners){
-                try {
-                    listener.onCreateActions(resources, this);
-                } catch (Exception e) {
-                    //must not fail
-                    PydevPlugin.log(e);
-                }
-            }
-        }
-
+        
+        // ----------------------------------------------------------------------------------------
+        // Offline action
+        action= new OfflineAction(resources, "Pyedit.ScriptEngine.", this); 
+        action.setActionDefinitionId("org.python.pydev.editor.actions.scriptEngine");
+        action.setId("org.python.pydev.editor.actions.scriptEngine");
+        setAction("PydevScriptEngine", action);
+        
+        
+        notifier.notifyOnCreateActions(resources);
     }
 
+
     protected void initializeKeyBindingScopes() {
         setKeyBindingScopes(new String[] { "org.python.pydev.ui.editor.scope" }); //$NON-NLS-1$
     }
@@ -553,18 +704,50 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
         return parser;
     }
 
-    public AbstractNode getPythonModel() {
-        return pythonModel;
+
+    /**
+     * Returns the status line manager of this editor.
+     * @return the status line manager of this editor
+     * @since 2.0
+     * 
+     * copied from superclass, as it is private there...
+     */
+    public IStatusLineManager getStatusLineManager() {
+
+        IEditorActionBarContributor contributor= getEditorSite().getActionBarContributor();
+        if (!(contributor instanceof EditorActionBarContributor))
+            return null;
+
+        IActionBars actionBars= ((EditorActionBarContributor) contributor).getActionBars();
+        if (actionBars == null)
+            return null;
+
+        return actionBars.getStatusLineManager();
     }
 
+    /**
+     * This is the 'offline' action
+     */
+    protected OfflineActionTarget fOfflineActionTarget;
+    
     /**
      * @return an outline view
      */
     public Object getAdapter(Class adapter) {
-        if (IContentOutlinePage.class.equals(adapter))
+        if (OfflineActionTarget.class.equals(adapter)) {
+            if (fOfflineActionTarget == null) {
+                IStatusLineManager manager= getStatusLineManager();
+                if (manager != null)
+                    fOfflineActionTarget= (getSourceViewer() == null ? null : new OfflineActionTarget(getSourceViewer(), manager, this));
+            }
+            return fOfflineActionTarget;
+        }
+
+        if (IContentOutlinePage.class.equals(adapter)){
             return new PyOutlinePage(this);
-        else
+        }else{
             return super.getAdapter(adapter);
+        }
     }
 
     /**
@@ -576,36 +759,56 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
         sourceViewer.revealRange(offset, length);
     }
 
-    /**
-     * Selects & reveals the model node
-     */
-    public void revealModelNode(AbstractNode node) {
+    public void revealModelNode(SimpleNode node) {
         if (node == null){
             return; // nothing to see here
         }
         
-        Location start = node.getStart();
-        Location end = node.getEnd();
-        
-        if(start == null || end == null){
-        	return;
+        IDocument document = getDocumentProvider().getDocument(getEditorInput());
+        if(document == null){
+            return;
         }
+        
+        int offset, length, endOffset;
+        
+        try {
+            PySelection selection = new PySelection(this);
+            offset = selection.getLineOffset(node.beginLine-1) + node.beginColumn-1;
+            int[] colLineEnd = NodeUtils.getColLineEnd(node);
+            
+            endOffset = selection.getLineOffset(colLineEnd[0]-1) + colLineEnd[1]-1;
+            length = endOffset - offset;
+            setSelection(offset, length);
+        } catch (Exception e) {
+            PydevPlugin.log(e);
+        }
+        
+    }
+    /**
+     * Selects & reveals the model node
+     */
+    public void revealModelNode(ASTEntry entry) {
+        if (entry == null){
+            return; // nothing to see here
+        }
+        
         IDocument document = getDocumentProvider().getDocument(getEditorInput());
         if(document == null){
         	return;
         }
         
-        int offset, length;
+        int offset, length, endOffset;
 
         try {
-            offset = start.toOffset(document);
-            length = end.toOffset(document) - offset;
-        } catch (BadLocationException e) {
-            PydevPlugin.log(IStatus.WARNING, "error trying to revealModelItem" + node.toString(), e);
-            return;
+            PySelection selection = new PySelection(this);
+            offset = selection.getLineOffset(entry.node.beginLine-1) + entry.node.beginColumn-1;
+            
+            endOffset = selection.getLineOffset(entry.endLine-1) + entry.endCol-1;
+            length = endOffset - offset;
+            setSelection(offset, length);
+        } catch (Exception e) {
+            PydevPlugin.log(e);
         }
-        
-        setSelection(offset, length);
     }
 
     /**
@@ -647,25 +850,11 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
             }
         }
 
-        
-        //delete the problem markers
-        try {
-            IResource res = (IResource) input.getAdapter(IResource.class);
-            if (res != null){
-                res.deleteMarkers(IMarker.PROBLEM, false, 1);
-            }
-        } catch (CoreException e) {
-            // What bad can come from removing markers? Ignore this exception
-            PydevPlugin.log(IStatus.WARNING, "Unexpected error removing markers", e);
-        }
-        
-        
         int lastLine = doc.getNumberOfLines();
         try {
             doc.getLineInformation(lastLine - 1);
             ast = root;
-            pythonModel = ModelMaker.createModel(root, doc, filePath);
-            fireModelChanged(pythonModel, ast);
+            fireModelChanged(ast);
         } catch (BadLocationException e1) {
             PydevPlugin.log(IStatus.WARNING, "Unexpected error getting document length. No model!", e1);
         }
@@ -687,7 +876,6 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
                 return;
             }
 
-            fileAdapter.deleteMarkers(IMarker.PROBLEM, false, 1);
             int errorStart;
             int errorEnd;
             int errorLine;
@@ -747,7 +935,7 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
             PydevPlugin.log(e1);
         } catch (BadLocationException e2) {
             // Whatever, could not create a marker. Swallow this one
-        	PydevPlugin.log(e2);
+        	//PydevPlugin.log(e2);
         }
     }
 
@@ -787,20 +975,23 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
      * 
      * @param root2
      */
-    protected void fireModelChanged(AbstractNode root, SimpleNode root2) {
+    protected void fireModelChanged(SimpleNode root) {
     	//create a copy, to avoid concurrent modifications
         for (IModelListener listener : new ArrayList<IModelListener>(modelListeners)) {
-        	listener.modelChanged(root, root2);
+        	listener.modelChanged(root);
 		}
     }
 
     /**
-     * @return
-     *  
+     * @return the python nature associated with this editor.
      */
     public IPythonNature getPythonNature() {
         IProject project = getProject();
-        return PythonNature.getPythonNature(project);
+        IPythonNature pythonNature = PythonNature.getPythonNature(project);
+        if(pythonNature == null){
+            pythonNature = PydevPlugin.getInfoForFile(getEditorFile()).o1;
+        }
+        return pythonNature;
     }
 
     protected void initializeEditor() {
@@ -816,5 +1007,63 @@ public class PyEdit extends PyEditProjection implements IPyEdit {
     public SimpleNode getAST() {
         return ast;
     }
+
+    Map<String, ActionInfo> onOfflineActionListeners = new HashMap<String, ActionInfo>();
+    public Collection<ActionInfo> getOfflineActionDescriptions(){
+        return onOfflineActionListeners.values();
+    }
+    public void addOfflineActionListener(String key, IAction action) {
+        onOfflineActionListeners.put(key, new ActionInfo(action, "not described", key, true));
+    }
+    public void addOfflineActionListener(String key, IAction action, String description, boolean needsEnter) {
+    	onOfflineActionListeners.put(key, new ActionInfo(action, description, key, needsEnter));
+	}
+    public boolean activatesAutomaticallyOn(String key){
+        ActionInfo info = onOfflineActionListeners.get(key);
+        if(info != null){
+            if(!info.needsEnter){
+                return true;
+            }
+        }
+        return false;
+    }
+    /**
+     * @return if an action was binded and was successfully executed
+     */
+	public boolean onOfflineAction(String requestedStr, OfflineActionTarget target) {
+		ActionInfo actionInfo = onOfflineActionListeners.get(requestedStr);
+        if(actionInfo == null){
+            target.statusError("No action was found binded to:"+requestedStr);
+            return false;
+            
+        }
+            
+        IAction action = actionInfo.action;
+		if(action == null){
+		    target.statusError("No action was found binded to:"+requestedStr);
+		    return false;
+        }
+        
+		try {
+			action.run();
+		} catch (Throwable e) {
+		    target.statusError("Exception raised when executing action:"+requestedStr+" - "+e.getMessage());
+			PydevPlugin.log(e);
+			return false;
+		}
+		return true;
+	}
+    
+    
+	public Font getFont(FontData descriptor) {
+        Font font = (Font) SWTResourceUtil.getFontTable().get(descriptor);
+        if (font == null) {
+            font = new Font(Display.getCurrent(), descriptor);
+            SWTResourceUtil.getFontTable().put(descriptor, font);
+        }
+        return font;
+    }
+
+
 }
 
