@@ -8,12 +8,11 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Set;
+import java.util.SortedMap;
 
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.jface.text.IDocument;
-import org.python.parser.SimpleNode;
-import org.python.parser.ast.FunctionDef;
 import org.python.pydev.core.FindInfo;
 import org.python.pydev.core.FullRepIterable;
 import org.python.pydev.core.ICodeCompletionASTManager;
@@ -33,6 +32,8 @@ import org.python.pydev.editor.codecompletion.revisited.modules.AbstractModule;
 import org.python.pydev.editor.codecompletion.revisited.modules.SourceModule;
 import org.python.pydev.editor.codecompletion.revisited.visitors.Definition;
 import org.python.pydev.parser.PyParser;
+import org.python.pydev.parser.jython.SimpleNode;
+import org.python.pydev.parser.jython.ast.FunctionDef;
 import org.python.pydev.parser.visitors.NodeUtils;
 
 public abstract class AbstractASTManager implements ICodeCompletionASTManager, Serializable {
@@ -98,7 +99,7 @@ public abstract class AbstractASTManager implements ICodeCompletionASTManager, S
         if (absoluteModule.endsWith(".")) {
             absoluteModule = absoluteModule.substring(0, absoluteModule.length() - 1);
         }
-        absoluteModule = absoluteModule.toLowerCase().trim();
+        //absoluteModule = absoluteModule.toLowerCase().trim();
 
         //set to hold the completion (no duplicates allowed).
         Set<IToken> set = new HashSet<IToken>();
@@ -124,11 +125,13 @@ public abstract class AbstractASTManager implements ICodeCompletionASTManager, S
      * @param set the set where the tokens should be added
      */
     protected void getAbsoluteImportTokens(String moduleToGetTokensFrom, Set<IToken> set, int type, boolean onlyFilesOnSameLevel) {
-        for (Iterator iter = Arrays.asList(modulesManager.getAllModules()).iterator(); iter.hasNext();) {
-            ModulesKey key = (ModulesKey) iter.next();
-
+    	SortedMap<ModulesKey,ModulesKey> modulesStartingWith = modulesManager.getAllModulesStartingWith(moduleToGetTokensFrom);
+    	Iterator<ModulesKey> itModules = modulesStartingWith.keySet().iterator();
+    	while(itModules.hasNext()){
+    		ModulesKey key = itModules.next();
+    		
             String element = key.name;
-			if (element.startsWith(moduleToGetTokensFrom)) {
+//			if (element.startsWith(moduleToGetTokensFrom)) { we don't check that anymore because we get all the modules starting with it already
                 if(onlyFilesOnSameLevel && key.file != null && key.file.isDirectory()){
                 	continue; // we only want those that are in the same directory, and not in other directories...
                 }
@@ -150,7 +153,7 @@ public abstract class AbstractASTManager implements ICodeCompletionASTManager, S
                 //and not xmlrpclib
                 //if we have xml token (not using the qualifier here) 
                 if (moduleToGetTokensFrom.length() != 0) {
-                    if (element.startsWith(".")) {
+                    if (element.length() > 0 && element.charAt(0) == ('.')) {
                         element = element.substring(1);
                         goForIt = true;
                     }
@@ -159,13 +162,13 @@ public abstract class AbstractASTManager implements ICodeCompletionASTManager, S
                 }
 
                 if (element.length() > 0 && goForIt) {
-                    String[] splitted = element.split("\\.");
+                    String[] splitted = FullRepIterable.dotSplit(element);
                     if (splitted.length > 0) {
                         //this is the completion
                         set.add(new ConcreteToken(splitted[0], "", "", moduleToGetTokensFrom, type));
                     }
                 }
-            }
+//            }
         }
     }
 
@@ -238,8 +241,8 @@ public abstract class AbstractASTManager implements ICodeCompletionASTManager, S
     public IToken[] getCompletionsForToken(IDocument doc, ICompletionState state) {
         IToken[] completionsForModule;
         try {
-	        Object[] obj = PyParser.reparseDocument(new PyParser.ParserInfo(doc, true, state.getNature(), state.getLine()));
-	        SimpleNode n = (SimpleNode) obj[0];
+            Tuple<SimpleNode, Throwable> obj = PyParser.reparseDocument(new PyParser.ParserInfo(doc, true, state.getNature(), state.getLine()));
+	        SimpleNode n = obj.o1;
 	        IModule module = AbstractModule.createModule(n);
         
             completionsForModule = getCompletionsForModule(module, state);
@@ -253,15 +256,27 @@ public abstract class AbstractASTManager implements ICodeCompletionASTManager, S
 
     
     
+    /**
+     * By default does not look for relative import
+     */
+    public IModule getModule(String name, IPythonNature nature, boolean dontSearchInit) {
+    	return modulesManager.getModule(name, nature, dontSearchInit, false);
+    }
 
     /**
      * This method returns the module that corresponds to the path passed as a parameter.
      * 
      * @param name
+     * @param lookingForRelative determines whether we're looking for a relative module (in which case we should
+     * not check in other places... only in the module)
      * @return the module represented by this name
      */
-    public IModule getModule(String name, IPythonNature nature, boolean dontSearchInit) {
-        return modulesManager.getModule(name, nature, dontSearchInit);
+    public IModule getModule(String name, IPythonNature nature, boolean dontSearchInit, boolean lookingForRelative) {
+    	if(lookingForRelative){
+    		return modulesManager.getRelativeModule(name, nature);
+    	}else{
+    		return modulesManager.getModule(name, nature, dontSearchInit);
+    	}
     }
 
     /**
@@ -278,7 +293,7 @@ public abstract class AbstractASTManager implements ICodeCompletionASTManager, S
         state2.setActivationToken (NodeUtils.getBuiltinType(state.getActivationToken()));
 
         if(state2.getActivationToken() != null){
-            IModule m = getModule("__builtin__", state.getNature(), false);
+            IModule m = getBuiltinMod(state.getNature());
             return m.getGlobalTokens(state2, this);
         }
         return null;
@@ -392,7 +407,7 @@ public abstract class AbstractASTManager implements ICodeCompletionASTManager, S
                 }
                 
                 //If it was still not found, go to builtins.
-                IModule builtinsMod = getModule("__builtin__", state.getNature(), false);
+                IModule builtinsMod = getBuiltinMod(state.getNature());
                 if(builtinsMod != null && builtinsMod != module){
 	                tokens = getCompletionsForModule( builtinsMod, state);
 	                if (tokens.length > 0){
@@ -480,7 +495,7 @@ public abstract class AbstractASTManager implements ICodeCompletionASTManager, S
             try {
                 Definition[] defs = s.findDefinition(state.getActivationToken(), state.getLine(), state.getCol(), state.getNature(), new ArrayList<FindInfo>());
                 for (int i = 0; i < defs.length; i++) {
-                    if(!(defs[0].ast instanceof FunctionDef)){
+                    if(!(defs[i].ast instanceof FunctionDef)){
                         //we might want to extend that later to check the return of some function...
                                 
 	                    ICompletionState copy = state.getCopy();
@@ -541,19 +556,49 @@ public abstract class AbstractASTManager implements ICodeCompletionASTManager, S
     }
 
     /**
-     * @see org.python.pydev.editor.codecompletion.revisited.ICodeCompletionASTManage#getBuiltinCompletions
+     * @return the builtin completions
      */
-    public List getBuiltinCompletions(ICompletionState state, List completions) {
-        IModule builtMod = getModule("__builtin__", state.getNature(), false);
-        if(builtMod != null){
-            IToken[] toks = builtMod.getGlobalTokens();
-            for (int i = 0; i < toks.length; i++) {
-                completions.add(toks[i]);
-            }
+    public List<IToken> getBuiltinCompletions(ICompletionState state, List<IToken> completions) {
+        IPythonNature nature = state.getNature();
+        IToken[] builtinCompletions = getBuiltinComps(nature);
+        if(builtinCompletions != null){
+			for (int i = 0; i < builtinCompletions.length; i++) {
+				completions.add(builtinCompletions[i]);
+			}
         }
         return completions;
+        
     }
 
+
+    /**
+     * @return the tokens in the builtins
+     */
+	protected IToken[] getBuiltinComps(IPythonNature nature) {
+		IToken[] builtinCompletions = nature.getBuiltinCompletions();
+		
+        if(builtinCompletions == null || builtinCompletions.length == 0){
+        	IModule builtMod = getBuiltinMod(nature);
+        	if(builtMod != null){
+        		builtinCompletions = builtMod.getGlobalTokens();
+        		nature.setBuiltinCompletions(builtinCompletions);
+        	}
+        }
+		return builtinCompletions;
+	}
+
+	/**
+	 * @return the module that represents the builtins
+	 */
+	protected IModule getBuiltinMod(IPythonNature nature) {
+		IModule mod = nature.getBuiltinMod();
+		if(mod == null){
+			mod = getModule("__builtin__", nature, false);
+			nature.setBuiltinMod(mod);
+		}
+		return mod;
+	}
+
     /**
      * @see org.python.pydev.editor.codecompletion.revisited.ICodeCompletionASTManage#getCompletionsForWildImport
      */
@@ -651,7 +696,8 @@ public abstract class AbstractASTManager implements ICodeCompletionASTManager, S
 	            final String modRep = importedModule.getRepresentation(); //this is its 'real' representation (alias) on the file (if it is from xxx import a as yyy, it is yyy)
 	            
 	            if(modRep.equals(tok)){
-	            	return findOnImportedMods(importedModule, tok, nature, activationToken, currentModuleName);
+                    String act = activationToken;
+	            	return findOnImportedMods(importedModule, tok, nature, act, currentModuleName);
 	            }
 	        }
         }   
@@ -661,6 +707,7 @@ public abstract class AbstractASTManager implements ICodeCompletionASTManager, S
     
     /**
      * Checks if some module can be resolved and returns the module it is resolved to (and to which token).
+     * 
      */
     protected Tuple<IModule, String> findOnImportedMods(IToken importedModule, String tok, IPythonNature nature, 
     		String activationToken, String currentModuleName) {
@@ -791,20 +838,24 @@ public abstract class AbstractASTManager implements ICodeCompletionASTManager, S
      * First it tries with the full path, and them removes a part of the final of
      * that path until it finds the module or the path is empty.
      * 
-     * @param rep
      * @param currentModuleName this is the module name (used to check validity for relative imports) -- not used if dontSearchInit is false
+     * if this parameter is not null, it means we're looking for a relative import. When checking for relative imports, 
+     * we should only check the modules that are directly under this project (so, we should not check the whole pythonpath for
+     * it, just direct modules) 
+     * 
      * @return tuple with found module and the String removed from the path in
      * order to find the module.
      */
     protected Tuple<IModule, String> findModuleFromPath(String rep, IPythonNature nature, boolean dontSearchInit, String currentModuleName){
         String tok = "";
-        IModule mod = getModule(rep, nature, dontSearchInit);
+        boolean lookingForRelative = currentModuleName != null;
+		IModule mod = getModule(rep, nature, dontSearchInit, lookingForRelative);
         String mRep = rep;
         int index;
         while(mod == null && (index = mRep.lastIndexOf('.')) != -1){
             tok = mRep.substring(index+1) + "."+tok;
             mRep = mRep.substring(0,index);
-            mod = getModule(mRep, nature, dontSearchInit);
+            mod = getModule(mRep, nature, dontSearchInit, lookingForRelative);
         }
         if (tok.endsWith(".")){
             tok = tok.substring(0, tok.length()-1); //remove last point if found.
