@@ -13,16 +13,16 @@ import java.io.ObjectOutputStream;
 import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.Collection;
-import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.SortedMap;
+import java.util.TreeMap;
 
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IResource;
-import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.jface.text.IDocument;
 import org.python.pydev.core.IModule;
@@ -30,37 +30,73 @@ import org.python.pydev.core.IModulesManager;
 import org.python.pydev.core.IPythonNature;
 import org.python.pydev.core.ModulesKey;
 import org.python.pydev.core.REF;
-import org.python.pydev.core.cache.Cache;
+import org.python.pydev.core.Tuple;
 import org.python.pydev.core.cache.LRUCache;
 import org.python.pydev.editor.codecompletion.PyCodeCompletion;
 import org.python.pydev.editor.codecompletion.revisited.modules.AbstractModule;
 import org.python.pydev.editor.codecompletion.revisited.modules.CompiledModule;
 import org.python.pydev.editor.codecompletion.revisited.modules.EmptyModule;
 import org.python.pydev.editor.codecompletion.revisited.modules.SourceModule;
-import org.python.pydev.plugin.PydevPlugin;
 
 /**
+ * This class manages the modules that are available
+ * 
  * @author Fabio Zadrozny
  */
 public abstract class ModulesManager implements IModulesManager, Serializable {
 
-	private final class ModulesManagerCache extends LRUCache<ModulesKey, AbstractModule> {
-		private ModulesManagerCache(int size) {
-			super(size);
-		}
+    /**
+     * This is a 'global' cache implementation, that can have at most n objects in
+     * the memory at any time.
+     */
+	private static final class ModulesManagerCache  {
+        /**
+         * Defines the maximun amount of modules that can be in the memory at any time (for all the managers)
+         */
+        private static final int MAX_NUMBER_OF_MODULES = 400;
 
+        private Object mutex;
+        
+        /**
+         * The access to the cache is synchronized
+         */
+		private LRUCache<Tuple<ModulesKey, ModulesManager>, AbstractModule> internalCache;
+
+        private ModulesManagerCache() {
+            mutex = this;
+            internalCache = new LRUCache<Tuple<ModulesKey, ModulesManager>, AbstractModule>(MAX_NUMBER_OF_MODULES);
+		}
+        
 		/**
 		 * Overriden so that if we do not find the key, we have the chance to create it.
 		 */
-		public AbstractModule getObj(ModulesKey key) {
-			AbstractModule obj = super.getObj(key);
-			if(obj == null && modulesKeys.containsKey(key)){
-				key = modulesKeys.get(key); //get the 'real' key
-				obj = AbstractModule.createEmptyModule(key.name, key.file);
-				this.add(key, obj);
-			}
-			return obj;
+		public AbstractModule getObj(ModulesKey key, ModulesManager modulesManager) {
+            synchronized (mutex) {
+    			Tuple<ModulesKey, ModulesManager> keyTuple = new Tuple<ModulesKey, ModulesManager>(key, modulesManager);
+                
+                AbstractModule obj = internalCache.getObj(keyTuple);
+    			if(obj == null && modulesManager.modulesKeys.containsKey(key)){
+    				key = modulesManager.modulesKeys.get(key); //get the 'real' key
+    				obj = AbstractModule.createEmptyModule(key.name, key.file);
+                    internalCache.add(keyTuple, obj);
+    			}
+    			return obj;
+            }
 		}
+
+        public void remove(ModulesKey key, ModulesManager modulesManager) {
+            synchronized (mutex) {
+                Tuple<ModulesKey, ModulesManager> keyTuple = new Tuple<ModulesKey, ModulesManager>(key, modulesManager);
+                internalCache.remove(keyTuple);
+            }
+        }
+
+        public void add(ModulesKey key, AbstractModule n, ModulesManager modulesManager) {
+            synchronized (mutex) {
+                Tuple<ModulesKey, ModulesManager> keyTuple = new Tuple<ModulesKey, ModulesManager>(key, modulesManager);
+                internalCache.add(keyTuple, n);
+            }
+        }
 	}
 
 	public ModulesManager(){
@@ -73,13 +109,14 @@ public abstract class ModulesManager implements IModulesManager, Serializable {
      * 
      * Implementation changed to contain a cache, so that it does not grow to much (some out of memo errors
      * were thrown because of the may size when having too many modules).
+     * 
+     * It is sorted so that we can get things in a 'subtree' faster
      */
-//    protected transient Map<ModulesKey, AbstractModule> modules = new HashMap<ModulesKey, AbstractModule>();
-	protected transient Map<ModulesKey, ModulesKey> modulesKeys = new HashMap<ModulesKey, ModulesKey>();
-	protected transient Cache<ModulesKey, AbstractModule> cache = createCache();
+	protected transient SortedMap<ModulesKey, ModulesKey> modulesKeys = new TreeMap<ModulesKey, ModulesKey>();
+	private static transient ModulesManagerCache cache = createCache();
     
-	protected Cache<ModulesKey, AbstractModule> createCache(){
-		return new ModulesManagerCache(300);
+	private static ModulesManagerCache createCache(){
+		return new ModulesManagerCache();
 	}
 	
     /**
@@ -99,8 +136,7 @@ public abstract class ModulesManager implements IModulesManager, Serializable {
      * Custom deserialization is needed.
      */
     private void readObject(ObjectInputStream aStream) throws IOException, ClassNotFoundException {
-    	cache = createCache();
-    	modulesKeys = new HashMap<ModulesKey, ModulesKey>();
+    	modulesKeys = new TreeMap<ModulesKey, ModulesKey>();
     	
         files = new HashSet<File>();
         aStream.defaultReadObject();
@@ -121,13 +157,13 @@ public abstract class ModulesManager implements IModulesManager, Serializable {
     private void writeObject(ObjectOutputStream aStream) throws IOException {
         aStream.defaultWriteObject();
         //write only the keys
-        aStream.writeObject(new HashSet(modulesKeys.keySet()));
+        aStream.writeObject(new HashSet<ModulesKey>(modulesKeys.keySet()));
     }
 
     /**
      * @param modules The modules to set.
      */
-    private void setModules(HashMap<ModulesKey, ModulesKey> keys) {
+    private void setModules(SortedMap<ModulesKey, ModulesKey> keys) {
         this.modulesKeys = keys;
     }
     
@@ -192,7 +228,7 @@ public abstract class ModulesManager implements IModulesManager, Serializable {
     private void changePythonPath(String pythonpath, final IProject project, IProgressMonitor monitor, 
     		List<String> pythonpathList, List<File> completions, List<String> fromJar, int total, String defaultSelectedInterpreter) {
 
-    	HashMap<ModulesKey, ModulesKey> keys = new HashMap<ModulesKey, ModulesKey>();
+    	SortedMap<ModulesKey, ModulesKey> keys = new TreeMap<ModulesKey, ModulesKey>();
         int j = 0;
 
         //now, create in memory modules for all the loaded files (empty modules).
@@ -363,7 +399,7 @@ public abstract class ModulesManager implements IModulesManager, Serializable {
      */
     protected void doRemoveSingleModule(ModulesKey key) {
         this.modulesKeys.remove(key);
-        this.cache.remove(key);
+        ModulesManager.cache.remove(key, this);
     }
 
     /**
@@ -373,9 +409,9 @@ public abstract class ModulesManager implements IModulesManager, Serializable {
      * @param key this is the key that should be added
      * @param n 
      */
-    protected void doAddSingleModule(final ModulesKey key, AbstractModule n) {
+    public void doAddSingleModule(final ModulesKey key, AbstractModule n) {
     	this.modulesKeys.put(key, key);
-        this.cache.add(key, n);
+        ModulesManager.cache.add(key, n, this);
     }
 
     /**
@@ -389,15 +425,21 @@ public abstract class ModulesManager implements IModulesManager, Serializable {
         return s;
     }
 
-    /**
-     * @return a Set of strings with all the modules.
-     */
-    public ModulesKey[] getAllModules() {
-        return (ModulesKey[]) this.modulesKeys.keySet().toArray(new ModulesKey[0]);
+    public SortedMap<ModulesKey,ModulesKey> getAllDirectModulesStartingWith(String strStartingWith) {
+    	if(strStartingWith.length() == 0){
+    		return modulesKeys;
+    	}
+    	ModulesKey startingWith = new ModulesKey(strStartingWith, null);
+    	ModulesKey endingWith = new ModulesKey(startingWith+"z", null);
+    	return modulesKeys.subMap(startingWith, endingWith);
+    }
+    
+    public SortedMap<ModulesKey,ModulesKey> getAllModulesStartingWith(String strStartingWith) {
+    	return getAllDirectModulesStartingWith(strStartingWith);
     }
     
     public ModulesKey[] getOnlyDirectModules() {
-        return getAllModules();
+    	return (ModulesKey[]) this.modulesKeys.keySet().toArray(new ModulesKey[0]);
     }
 
     /**
@@ -426,22 +468,50 @@ public abstract class ModulesManager implements IModulesManager, Serializable {
         //be source modules, but they are so hacked that it is almost impossible to get useful information
         //from them).
         String[] builtins = getBuiltins();
+        if(builtins == null){
+        	//still on startup
+        	return null;
+        }
         
+        boolean foundStartingWithBuiltin = false;
         for (int i = 0; i < builtins.length; i++) {
-            if (name.equals(builtins[i])) {
-                n = cache.getObj(new ModulesKey(name, null));
-                if(n == null || n instanceof EmptyModule || n instanceof SourceModule){ //still not created or not defined as compiled module (as it should be)
-                    n = new CompiledModule(name, PyCodeCompletion.TYPE_BUILTIN, nature.getAstManager());
-                    doAddSingleModule(new ModulesKey(n.getName(), null), n);
+            String forcedBuiltin = builtins[i];
+            if (name.startsWith(forcedBuiltin)) {
+                if(name.length() > forcedBuiltin.length() && name.charAt(forcedBuiltin.length()) == '.'){
+                	foundStartingWithBuiltin = true;
+                	n = cache.getObj(new ModulesKey(name, null), this);
+                	if(n == null && dontSearchInit == false){
+                		n = cache.getObj(new ModulesKey(name+".__init__", null), this);
+                	}
+                	if(n instanceof EmptyModule || n instanceof SourceModule){ //it is actually found as a source module, so, we have to 'coerce' it to a compiled module
+                		n = new CompiledModule(name, PyCodeCompletion.TYPE_BUILTIN, nature.getAstManager());
+                		doAddSingleModule(new ModulesKey(n.getName(), null), n);
+                		return n;
+                	}
+                }
+                
+                if(name.equals(forcedBuiltin)){
+                    n = cache.getObj(new ModulesKey(name, null), this);
+                    if(n == null || n instanceof EmptyModule || n instanceof SourceModule){ //still not created or not defined as compiled module (as it should be)
+                        n = new CompiledModule(name, PyCodeCompletion.TYPE_BUILTIN, nature.getAstManager());
+                        doAddSingleModule(new ModulesKey(n.getName(), null), n);
+                        return n;
+                    }
+                }
+                if(n instanceof CompiledModule){
+                	return n;
                 }
             }
         }
+        if(foundStartingWithBuiltin){
+        	return null;
+        }
 
 
         if(n == null){
             if(!dontSearchInit){
                 if(n == null){
-                    n = cache.getObj(new ModulesKey(name + ".__init__", null));
+                    n = cache.getObj(new ModulesKey(name + ".__init__", null), this);
                     if(n != null){
                         name += ".__init__";
                     }
@@ -449,7 +519,7 @@ public abstract class ModulesManager implements IModulesManager, Serializable {
             }
             if (n == null) {
             	ModulesKey key = new ModulesKey(name, null);
-            	n = cache.getObj(key);
+            	n = cache.getObj(key, this);
             }
         }
 
@@ -511,15 +581,8 @@ public abstract class ModulesManager implements IModulesManager, Serializable {
      * @see org.python.pydev.core.IProjectModulesManager#resolveModule(org.eclipse.core.resources.IResource, org.eclipse.core.resources.IProject)
      */
     public String resolveModule(IResource member, IProject container) {
-        IPath location = PydevPlugin.getLocation(member.getFullPath(), container);
-        if(location == null){
-            //not in workspace?... maybe it was removed, so, do nothing, but let the user know about it
-            PydevPlugin.log(getResolveModuleErr(member));
-            return null;
-        }else{
-            File inOs = new File(location.toOSString());
-            return resolveModule(REF.getFileAbsolutePath(inOs));
-        }
+        File inOs = member.getRawLocation().toFile();
+        return resolveModule(REF.getFileAbsolutePath(inOs));
     }
 
     protected String getResolveModuleErr(IResource member) {
@@ -538,4 +601,5 @@ public abstract class ModulesManager implements IModulesManager, Serializable {
     public List<String> getPythonPath(){
         return new ArrayList<String>(pythonPathHelper.pythonpath);
     }
+
 }
