@@ -5,16 +5,10 @@
  */
 package org.python.pydev.editor.codecompletion.revisited;
 
-import java.io.BufferedReader;
 import java.io.File;
 import java.io.FileFilter;
-import java.io.FileInputStream;
-import java.io.FileNotFoundException;
-import java.io.IOException;
-import java.io.InputStreamReader;
-import java.io.Reader;
+import java.io.FilenameFilter;
 import java.io.Serializable;
-import java.io.StringReader;
 import java.util.ArrayList;
 import java.util.Enumeration;
 import java.util.HashSet;
@@ -28,9 +22,9 @@ import org.eclipse.core.resources.IFile;
 import org.eclipse.core.runtime.IPath;
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.core.runtime.Path;
-import org.eclipse.jface.text.IDocument;
 import org.python.pydev.core.FullRepIterable;
 import org.python.pydev.core.REF;
+import org.python.pydev.core.docutils.StringUtils;
 import org.python.pydev.plugin.PydevPlugin;
 
 /**
@@ -54,20 +48,15 @@ public class PythonPathHelper implements Serializable{
      * @param acceptPoint says if we can have dots in the str that has the path to be analyzed
      * @return a trimmed string with all the '\' converted to '/'
      */
-    public String getDefaultPathStr(String str, boolean acceptPoint){
-        if(acceptPoint == false && str.indexOf(".") == 0){ //cannot start with a dot
-            throw new RuntimeException("The pythonpath can only have absolute paths (cannot start with '.', therefore, the path: '"+str+"' is not valid.");
-        }
-        File file = new File(str);
-        if(file.exists()){
-            str = REF.getFileAbsolutePath(file);
-        }
-        return str.trim().replaceAll("\\\\","/");
+    public String getDefaultPathStr(String str){
+    	//this check is no longer done... could result in other problems
+		// if(acceptPoint == false && str.indexOf(".") == 0){ //cannot start with a dot
+		// 		throw new RuntimeException("The pythonpath can only have absolute paths (cannot start with '.', therefore, the path: '"+str+"' is not valid.");
+		// }
+    	return StringUtils.replaceAllSlashes(str.trim());
     }
     
-    public String getDefaultPathStr(String str){
-        return getDefaultPathStr(str, false);
-    }
+    
     
     /**
      * This method returns all modules that can be obtained from a root File.
@@ -162,28 +151,53 @@ public class PythonPathHelper implements Serializable{
         return false;
     }
     
+    private final static String[] DOTTED_VALID_SOURCE_FILES = new String[]{".py", ".pyw"};
+    private final static String[] VALID_SOURCE_FILES = new String[]{"py", "pyw"};
+    
+    public final static String[] getDottedValidSourceFiles() {
+    	return DOTTED_VALID_SOURCE_FILES;
+    }
+    
+    public final static String[] getValidSourceFiles() {
+    	return VALID_SOURCE_FILES;
+    }
+    
+    public final static String getDefaultDottedPythonExtension(){
+    	return ".py";
+    }
+    
     /**
-     * @param path
-     * @return if the path passed belongs to a valid python source file
+     * @return if the path passed belongs to a valid python source file (checks for the extension)
      */
     public static boolean isValidSourceFile(String path) {
         path = path.toLowerCase();
-        return path.endsWith(".py") || path.endsWith(".pyw");
+        for(String end : getDottedValidSourceFiles()){
+        	if(path.endsWith(end)){
+        		return true;
+        	}
+        }
+        return false;
     }
     
+    /**
+     * @return whether an IFile is a valid source file given its extension
+     */
     public static boolean isValidSourceFile(IFile file) {
         String ext = file.getFileExtension();
         if(ext == null){ // no extension
         	return false;
         }
 		ext = ext.toLowerCase();
-        return ext.equals("py") || ext.equals("pyw");
+		for(String end : getValidSourceFiles()){
+			if(ext.equals(end)){
+				return true;
+			}
+		}
+        return false;
     }
 
     
     /**
-     * 
-     * @param path
      * @return if the paths maps to a valid python module (depending on its extension).
      */
     public static boolean isValidFileMod(String path){
@@ -210,16 +224,17 @@ public class PythonPathHelper implements Serializable{
      * @param module - this is the full path of the module. Only for directories or py,pyd,dll,pyo files.
      * @return a String with the module that the file or folder should represent. E.g.: compiler.ast
      */
-    public String resolveModule(String fullPath, boolean requireFileToExist){
-        fullPath = getDefaultPathStr(fullPath, true);
-        File moduleFile = new File(fullPath);
+    public String resolveModule(String fullPath, final boolean requireFileToExist){
+        fullPath = getDefaultPathStr(fullPath);
+        final File moduleFile = new File(fullPath);
         
         if(requireFileToExist){
 	        if(moduleFile.exists() == false){
 	            return null;
 	        }
         }
-        if(moduleFile.isFile()){
+        boolean isFile = moduleFile.isFile();
+		if(isFile){
             
             if(isValidFileMod(REF.getFileAbsolutePath(moduleFile)) == false){
                 return null;
@@ -248,9 +263,9 @@ public class PythonPathHelper implements Serializable{
                         
                         //this means that more than 1 module is specified, so, in order to get it,
                         //we have to go and see if all the folders to that module have __init__.py in it...
-                        String[] modulesParts = s.split("\\.");
+                        String[] modulesParts = FullRepIterable.dotSplit(s);
                         
-                        if(modulesParts.length > 1 && moduleFile.isFile()){
+                        if(modulesParts.length > 1 && isFile){
                             String[] t = new String[modulesParts.length -1];
                             
                             for (int i = 0; i < modulesParts.length-1; i++) {
@@ -268,17 +283,16 @@ public class PythonPathHelper implements Serializable{
                         //chars as the full path passed in.
                         boolean isValid = true;
                         for (int i = 0; i < modulesParts.length && root != null; i++) {
-                            
+                        	root = new File(REF.getFileAbsolutePath(root) + "/" + modulesParts[i]);
+                        	
                             //check if file is in root...
                             if(isValidFileMod(modulesParts[i])){
-                                root = new File(REF.getFileAbsolutePath(root) + "/" + modulesParts[i]);
                                 if(root.exists() && root.isFile()){
                                     break;
                                 }
                                 
                             }else{
                                 //this part is a folder part... check if it is a valid module (has init).
-	                            root = new File(REF.getFileAbsolutePath(root) + "/" + modulesParts[i]);
 	                            if(isFileOrFolderWithInit(root) == false){
 	                                isValid = false;
 	                                break;
@@ -287,7 +301,7 @@ public class PythonPathHelper implements Serializable{
                             }                            
                         }
                         if(isValid){
-                            if(moduleFile.isFile()){
+                            if(isFile){
                                 s = stripExtension(s);
                             }else if(moduleFile.exists() == false){
                             	//ok, it does not exist, so isFile will not work, let's just check if it is
@@ -300,7 +314,7 @@ public class PythonPathHelper implements Serializable{
                         }
                     }else{
                         //simple part, we don't have to go into subfolders to check validity...
-                        if(moduleFile.isFile()){
+                        if(isFile){
                             throw new RuntimeException("This should never happen... if it is a file, it always has a dot, so, this should not happen...");
                         }else if (moduleFile.isDirectory() && isFileOrFolderWithInit(moduleFile) == false){
                             return null;
@@ -345,32 +359,36 @@ public class PythonPathHelper implements Serializable{
     }
 
     /**
-     * @param root
-     * @param string
-     * @return
+     * @param root this is the folder we're checking
+     * @return true if it is a folder with an __init__ python file
      */
     private boolean isFileOrFolderWithInit(File root) {
-        if(root.isDirectory() == false){
-            return true;
+        //check for an __init__ in a dir (we do not check if it is a file, becase if it is, it should return null)
+        String[] items = root.list(new FilenameFilter(){
+
+			public boolean accept(File dir, String name) {
+				if(isValidInitFile(name)){
+					return true;
+				}
+				return false;
+			}
+        	
+        });
+        if(items == null || items.length < 1){
+        	return false;
         }
         
-        File[] files = root.listFiles();
-        
-        if(files == null){
-            return false;
-        }
-        
-        //check for an __init__.py
-        String[] items = root.list();
-        for (int j = 0; j < items.length; j++) {
-            if(items[j].toLowerCase().equals("__init__.py")){
-                return true;
-            }
-        }
-        
-        return false;
+        return true;
     }
 
+    /**
+     * @param item the file we want to check
+     * @return true if the file is a valid __init__ file
+     */
+	public static boolean isValidInitFile(String item) {
+		return item.toLowerCase().indexOf("__init__.") != -1 && isValidSourceFile(item);
+	}
+
     /**
      * @param s
      * @return
@@ -408,78 +426,4 @@ public class PythonPathHelper implements Serializable{
         }
 	}
 
-    public static String getPythonFileEncoding(IDocument doc) {
-        Reader inputStreamReader = new StringReader(doc.get());
-        return getPythonFileEncoding(inputStreamReader);
-    }
-    
-    /**
-     * @param f
-     * @return
-     */
-    public static String getPythonFileEncoding(File f) {
-        try {
-            Reader inputStreamReader = new InputStreamReader(new FileInputStream(f));
-            return getPythonFileEncoding(inputStreamReader);
-        } catch (FileNotFoundException e) {
-            return null;
-        }
-    }
-
-    /**
-     * @param inputStreamReader
-     */
-    private static String getPythonFileEncoding(Reader inputStreamReader) {
-        String ret = null;
-        BufferedReader reader = new BufferedReader(inputStreamReader);
-        try{
-            
-            String l1 = reader.readLine();
-            String l2 = reader.readLine();
-            
-            String lEnc = null;
-            //encoding must be specified in first or second line...
-            if (l1 != null && l1.toLowerCase().indexOf("coding") != -1){
-                lEnc = l1; 
-            }
-            else if (l2 != null && l2.toLowerCase().indexOf("coding") != -1){
-                lEnc = l2; 
-            }
-            else{
-                ret = null;
-            }
-            
-            if(lEnc != null){
-                lEnc = lEnc.trim();
-                if(lEnc.length() == 0){
-                    ret = null;
-                    
-                }else if(lEnc.charAt(0) == '#'){ //it must be a comment line
-                    
-                    //ok, the encoding line is in lEnc
-                    lEnc = lEnc.substring(lEnc.indexOf("coding")+6);
-                    
-                    char c;
-                    while(lEnc.length() > 0 && ((c = lEnc.charAt(0)) == ' ' || c == ':' || c == '=')) {
-                        lEnc = lEnc.substring(1);
-                    }
-    
-                    StringBuffer buffer = new StringBuffer();
-                    while(lEnc.length() > 0 && ((c = lEnc.charAt(0)) != ' ' || c == '-' || c == '*')) {
-                        
-                        buffer.append(c);
-                        lEnc = lEnc.substring(1);
-                    }
-    
-                    ret = buffer.toString().trim();
-                }
-            }
-        } catch (IOException e) {
-            e.printStackTrace();
-        }finally{
-            try {reader.close();} catch (IOException e1) {}
-        }
-        return ret;
-    }
-
 }
