@@ -11,12 +11,6 @@ import java.util.Arrays;
 import java.util.Iterator;
 import java.util.List;
 
-import org.python.parser.SimpleNode;
-import org.python.parser.ast.Attribute;
-import org.python.parser.ast.ClassDef;
-import org.python.parser.ast.FunctionDef;
-import org.python.parser.ast.Name;
-import org.python.parser.ast.Str;
 import org.python.pydev.core.FindInfo;
 import org.python.pydev.core.FullRepIterable;
 import org.python.pydev.core.ICodeCompletionASTManager;
@@ -27,6 +21,7 @@ import org.python.pydev.core.IPythonNature;
 import org.python.pydev.core.IToken;
 import org.python.pydev.core.REF;
 import org.python.pydev.core.Tuple;
+import org.python.pydev.core.structure.FastStack;
 import org.python.pydev.editor.codecompletion.revisited.AbstractToken;
 import org.python.pydev.editor.codecompletion.revisited.CompletionRecursionException;
 import org.python.pydev.editor.codecompletion.revisited.CompletionState;
@@ -35,6 +30,15 @@ import org.python.pydev.editor.codecompletion.revisited.visitors.Definition;
 import org.python.pydev.editor.codecompletion.revisited.visitors.FindDefinitionModelVisitor;
 import org.python.pydev.editor.codecompletion.revisited.visitors.FindScopeVisitor;
 import org.python.pydev.editor.codecompletion.revisited.visitors.GlobalModelVisitor;
+import org.python.pydev.editor.codecompletion.revisited.visitors.Scope;
+import org.python.pydev.parser.jython.SimpleNode;
+import org.python.pydev.parser.jython.ast.Assign;
+import org.python.pydev.parser.jython.ast.Attribute;
+import org.python.pydev.parser.jython.ast.ClassDef;
+import org.python.pydev.parser.jython.ast.FunctionDef;
+import org.python.pydev.parser.jython.ast.ImportFrom;
+import org.python.pydev.parser.jython.ast.Name;
+import org.python.pydev.parser.jython.ast.Str;
 import org.python.pydev.parser.visitors.NodeUtils;
 import org.python.pydev.plugin.PydevPlugin;
 
@@ -54,6 +58,8 @@ import org.python.pydev.plugin.PydevPlugin;
  */
 public class SourceModule extends AbstractModule {
 
+    public static boolean TESTING = false;
+    
     /**
      * This is the abstract syntax tree based on the jython parser output.
      */
@@ -147,65 +153,101 @@ public class SourceModule extends AbstractModule {
 
     
     /**
-     * @see org.python.pydev.editor.codecompletion.revisited.modules.AbstractModule#getGlobalTokens(java.lang.String)
+     * @see org.python.pydev.core.IModule#getGlobalTokens(org.python.pydev.core.ICompletionState, org.python.pydev.core.ICodeCompletionASTManager)
      */
-    public IToken[] getGlobalTokens(ICompletionState state, ICodeCompletionASTManager manager) {
+    public IToken[] getGlobalTokens(ICompletionState initialState, ICodeCompletionASTManager manager) {
         IToken[] t = getTokens(GlobalModelVisitor.GLOBAL_TOKENS);
         
         if(t instanceof SourceToken[]){
 	        SourceToken[] tokens = (SourceToken[]) t;
 	        for (int i = 0; i < tokens.length; i++) {
-	            if(tokens[i].getRepresentation().equals(state.getActivationToken())){
-	                
-	                SimpleNode a = tokens[i].getAst();
-	                    
-                    //COMPLETION: get the completions for the whole hierarchy if this is a class!!
-                    List modToks = new ArrayList(Arrays.asList(GlobalModelVisitor.getTokens(a, GlobalModelVisitor.INNER_DEFS, name)));
+	            String activationToken = initialState.getActivationToken();
+                SourceToken token = tokens[i];
+                String rep = token.getRepresentation();
+                
+                SimpleNode ast = token.getAst();
+                
+                if(activationToken.length() > rep.length() && activationToken.startsWith(rep)){
+                    //we need this thing to work correctly for nested modules...
+                    //some tests are available at: PythonCompletionTestWithoutBuiltins.testDeepNestedXXX
                     
-                    try {
-                        if (a instanceof ClassDef) {
-                            ClassDef c = (ClassDef) a;
-                            for (int j = 0; j < c.bases.length; j++) {
-                                if (c.bases[j] instanceof Name) {
-                                    Name n = (Name) c.bases[j];
-                                    String base = n.id;
-                                    //An error in the programming might result in an error.
-                                    //
-                                    //e.g. The case below results in a loop.
-                                    //
-                                    //class A(B):
-                                    //    
-                                    //    def a(self):
-                                    //        pass
-                                    //        
-                                    //class B(A):
-                                    //    
-                                    //    def b(self):
-                                    //        pass
-                                    state = state.getCopy();
-                                    state.setActivationToken(base);
-
-                                    state.checkMemory(this, base);
-
-                                    final IToken[] comps = manager.getCompletionsForModule(this, state);
-                                    modToks.addAll(Arrays.asList(comps));
-                                } else if (c.bases[j] instanceof Attribute) {
-                                    Attribute attr = (Attribute) c.bases[j];
-                                    String s = NodeUtils.getFullRepresentationString(attr);
-
-                                    state = state.getCopy();
-                                    state.setActivationToken(s);
-                                    final IToken[] comps = manager.getCompletionsForModule(this, state);
-                                    modToks.addAll(Arrays.asList(comps));
+                    int iActTok = 0;
+                    String[] actToks = FullRepIterable.dotSplit(activationToken);
+                    if(actToks[iActTok].equals(rep)){
+                        //System.out.println("Now we have to find act..."+act+"(which is a definition of:"+rep+")");
+                        try {
+                            Definition[] definitions;
+                            String value = activationToken;
+                            String initialValue=null;
+                            while(true){
+                                if(value.equals(initialValue)){
+                                    break;
                                 }
+                                initialValue = value;
+                                if(iActTok > actToks.length){
+                                    break; //unable to find it
+                                }
+                                definitions = findDefinition(value, token.getLineDefinition(), token.getColDefinition(), manager.getNature(), new ArrayList<FindInfo>());
+                                if(definitions.length == 1){
+                                    Definition d = definitions[0];
+                                    if(d.ast instanceof Assign){
+                                        Assign assign = (Assign) d.ast;
+                                        value = NodeUtils.getRepresentationString(assign.value);
+                                        definitions = findDefinition(value, d.line, d.col, manager.getNature(), new ArrayList<FindInfo>());
+                                    }else if(d.ast instanceof ClassDef){
+                                        IToken[] toks = (IToken[]) getToks(initialState, manager, d.ast).toArray(new IToken[0]);
+                                        if(iActTok == actToks.length-1){
+                                            return toks;
+                                        }
+                                        value = d.value;
+                                        
+                                    }else if (d.ast instanceof Name){
+                                        ClassDef classDef = d.scope.getClassDef();
+                                        if(classDef != null){
+                                        	FindDefinitionModelVisitor visitor = new FindDefinitionModelVisitor(actToks[actToks.length-1], d.line, d.col, d.module);
+	                                        classDef.accept(visitor);
+	                                        if(visitor.definitions.size() == 0){
+	                                        	return new IToken[0];
+	                                        }
+	                                        d = visitor.definitions.get(0);
+	                                        value = d.value;
+	                                        if(d instanceof AssignDefinition){
+	                                            return getValueCompletions(initialState, manager, value);
+	                                        }
+                                        }else{
+                                        	if(d.module instanceof SourceModule){
+                                        		FullRepIterable.joinFirstParts(actToks);
+                                        		SourceModule m = (SourceModule) d.module;
+                                        		Definition[] definitions2 = m.findDefinition(FullRepIterable.joinFirstParts(actToks), d.line, d.col,manager.getNature(), null);
+                                        		if(definitions2.length == 0){
+                                        			return new IToken[0];
+                                        		}
+                                        		d = definitions2[0];
+                                        		value = d.value+"."+actToks[actToks.length-1];
+                                        		if(d instanceof AssignDefinition){
+                                        			return ((SourceModule)d.module).getValueCompletions(initialState, manager, value);
+                                        		}
+                                        	}
+                                        }
+                                        
+                                    }else if ((d.ast == null && d.module != null) || d.ast instanceof ImportFrom){
+                                        return getValueCompletions(initialState, manager, value);
+                                        
+                                    }else{
+                                        break;
+                                    }
+                                }else{
+                                    return getValueCompletions(initialState, manager, value);
+                                }
+                                iActTok++;
                             }
-
+                        } catch (CompletionRecursionException e) {
+                        } catch (Exception e) {
+                            PydevPlugin.log(e);
                         }
-                    } catch (CompletionRecursionException e) {
-                        // let's return what we have so far...
                     }
-                    
-                    return (IToken[]) modToks.toArray(new IToken[0]);
+                } else if(rep.equals(activationToken)){
+                    return (IToken[]) getToks(initialState, manager, ast).toArray(new IToken[0]);
 	            }
 	        }
         }else{
@@ -214,7 +256,81 @@ public class SourceModule extends AbstractModule {
         return new IToken[0];
     }
 
-    public Definition[] findDefinition(String rep, int line, int col, IPythonNature nature, List<FindInfo> lFindInfo) throws Exception{
+    /**
+     * @param initialState
+     * @param manager
+     * @param value
+     * @return
+     */
+    private IToken[] getValueCompletions(ICompletionState initialState, ICodeCompletionASTManager manager, String value) {
+        initialState.checkFindMemory(this, value);
+        ICompletionState copy = initialState.getCopy();
+        copy.setActivationToken(value);
+        IToken[] completionsForModule = manager.getCompletionsForModule(this, copy);
+        return completionsForModule;
+    }
+
+    /**
+     * @param initialState
+     * @param manager
+     * @param ast
+     * @return
+     */
+    private List<IToken> getToks(ICompletionState initialState, ICodeCompletionASTManager manager, SimpleNode ast) {
+        List<IToken> modToks = modToks = new ArrayList<IToken>(Arrays.asList(GlobalModelVisitor.getTokens(ast, GlobalModelVisitor.INNER_DEFS, name)));//name = moduleName
+        
+        try {
+            //COMPLETION: get the completions for the whole hierarchy if this is a class!!
+            ICompletionState state;
+            if (ast instanceof ClassDef) {
+                ClassDef c = (ClassDef) ast;
+                for (int j = 0; j < c.bases.length; j++) {
+                    if (c.bases[j] instanceof Name) {
+                        Name n = (Name) c.bases[j];
+                        String base = n.id;
+                        //An error in the programming might result in an error.
+                        //
+                        //e.g. The case below results in a loop.
+                        //
+                        //class A(B):
+                        //    
+                        //    def a(self):
+                        //        pass
+                        //        
+                        //class B(A):
+                        //    
+                        //    def b(self):
+                        //        pass
+                        state = initialState.getCopy();
+                        state.setActivationToken(base);
+
+                        state.checkMemory(this, base);
+
+                        final IToken[] comps = manager.getCompletionsForModule(this, state);
+                        modToks.addAll(Arrays.asList(comps));
+                    } else if (c.bases[j] instanceof Attribute) {
+                        Attribute attr = (Attribute) c.bases[j];
+                        String s = NodeUtils.getFullRepresentationString(attr);
+
+                        state = initialState.getCopy();
+                        state.setActivationToken(s);
+                        final IToken[] comps = manager.getCompletionsForModule(this, state);
+                        modToks.addAll(Arrays.asList(comps));
+                    }
+                }
+
+            }
+        } catch (CompletionRecursionException e) {
+            // let's return what we have so far...
+        }
+        return modToks;
+    }
+
+    @SuppressWarnings("unchecked")
+	public Definition[] findDefinition(String rep, int line, int col, IPythonNature nature, List<FindInfo> lFindInfo) throws Exception{
+    	if(lFindInfo == null){
+    		lFindInfo = new ArrayList<FindInfo>();
+    	}
         //the line passed in starts at 1 and the lines for the visitor start at 0
         ArrayList<Definition> toRet = new ArrayList<Definition>();
         FindInfo info = new FindInfo();
@@ -225,7 +341,6 @@ public class SourceModule extends AbstractModule {
         if (ast != null){
         	ast.accept(scopeVisitor);
         }
-
         
         //this visitor checks for assigns for the token
         FindDefinitionModelVisitor visitor = new FindDefinitionModelVisitor(rep, line, col+1, this);
@@ -237,14 +352,19 @@ public class SourceModule extends AbstractModule {
         	//ok, it is an assign, so, let's get it
 
             for (Iterator iter = visitor.definitions.iterator(); iter.hasNext();) {
-	            AssignDefinition element = (AssignDefinition) iter.next();
-	            if(element.target.startsWith("self") == false){
-		            if(element.scope.isOuterOrSameScope(scopeVisitor.scope)){
-		                toRet.add(element);
-		            }
-	            }else{
-		            toRet.add(element);
-	            }
+	            Object next = iter.next();
+                if(next instanceof AssignDefinition){
+                    AssignDefinition element = (AssignDefinition) next;
+    	            if(element.target.startsWith("self") == false){
+    		            if(element.scope.isOuterOrSameScope(scopeVisitor.scope)){
+    		                toRet.add(element);
+    		            }
+    	            }else{
+    		            toRet.add(element);
+    	            }
+                }else{
+                    toRet.add((Definition) next);
+                }
 	        }
             return (Definition[]) toRet.toArray(new Definition[0]);
         }
@@ -292,10 +412,19 @@ public class SourceModule extends AbstractModule {
 					if(token.getRepresentation().equals(withoutSelf)){
 						String parentPackage = token.getParentPackage();
 						IModule module = nature.getAstManager().getModule(parentPackage, nature, true);
-						if(module != null && token instanceof SourceToken){
+                        
+						if(token instanceof SourceToken && (module != null || this.name.equals(parentPackage))){
+                            if(module == null){
+                                module = this;
+                            }
+                            
 			                SimpleNode ast2 = ((SourceToken)token).getAst();
 							Tuple<Integer, Integer> def = getLineColForDefinition(ast2);
-							return new Definition[]{new Definition(def.o1, def.o2, token.getRepresentation(), ast2, null, module)};
+							FastStack<SimpleNode> stack = new FastStack<SimpleNode>();
+							stack.add(classDef);
+							Scope scope = new Scope(stack);
+							return new Definition[]{new Definition(def.o1, def.o2, token.getRepresentation(), ast2, scope, module)};
+                            
 						}else{
 							return new Definition[0];
 						}
@@ -319,12 +448,7 @@ public class SourceModule extends AbstractModule {
             //ok, we have to check the compiled module
             tok = o.o2;
             if (tok == null || tok.length() == 0 ){
-                if(o.o1.getFile() == null){
-                    return new Definition[0];
-                    
-                }else{
-                    return new Definition[]{new Definition(1,1,"",null,null,o.o1)};
-                }
+                return new Definition[]{new Definition(1,1,"",null,null,o.o1)};
             }else{
                 return (Definition[]) o.o1.findDefinition(tok, 0, 0, nature, lFindInfo);
             }
@@ -401,31 +525,63 @@ public class SourceModule extends AbstractModule {
     		tokens = getGlobalTokens();
     	}
         for (IToken token : tokens) {
-            if(token.getRepresentation().equals(rep) && token instanceof SourceToken){
-            	//ok, we found it
-                SimpleNode a = ((SourceToken)token).getAst();
-                
-                //this is just to get its scope...
-                FindScopeVisitor scopeVisitor = new FindScopeVisitor(a.beginLine, a.beginColumn);
-                if (ast != null){
+            boolean sameRep = token.getRepresentation().equals(rep);
+            if(sameRep){
+                if(token instanceof SourceToken){
+                	//ok, we found it
+                    SimpleNode a = ((SourceToken)token).getAst();
+                    
+                    //this is just to get its scope...
+                    FindScopeVisitor scopeVisitor = new FindScopeVisitor(a.beginLine, a.beginColumn);
+                    if (ast != null){
+                        try {
+                            ast.accept(scopeVisitor);
+                        } catch (Exception e) {
+                            PydevPlugin.log(e);
+                        }
+                    }
+                    Tuple<Integer, Integer> def = getLineColForDefinition(a);
+                    
+                    String parentPackage = token.getParentPackage();
+                    IModule module = this;
+                    if(nature != null){
+                    	IModule mod = nature.getAstManager().getModule(parentPackage, nature, true);
+                    	if(mod != null){
+                    		module = mod;
+                    	}
+                    }
+                    //line, col
+                    return new Definition(def.o1, def.o2, tok, a, scopeVisitor.scope, module);
+                }else{
+                    CompiledToken comp = (CompiledToken) token;
+                    String parentPackage = comp.getParentPackage();
+                    FullRepIterable iterable = new FullRepIterable(parentPackage, true);
+                    
+                    IModule module = null;
+                    for(String modName: iterable){
+                        module = nature.getAstManager().getModule(modName, nature, true);
+                        if(module != null){
+                            break;
+                        }
+                    }
+                    if(module == null){
+                        return null;
+                    }
+                    int length = module.getName().length();
+                    String finalRep = "";
+                    if(parentPackage.length() > length){
+                        finalRep = parentPackage.substring(length + 1)+'.';
+                    }
+                    finalRep += comp.getRepresentation();
                     try {
-                        ast.accept(scopeVisitor);
+                        IDefinition[] definitions = module.findDefinition(finalRep, -1, -1, nature, new ArrayList<FindInfo>());
+                        if(definitions.length > 0){
+                            return (Definition) definitions[0];
+                        }
                     } catch (Exception e) {
-                        PydevPlugin.log(e);
+                        throw new RuntimeException(e);
                     }
                 }
-                Tuple<Integer, Integer> def = getLineColForDefinition(a);
-                
-                String parentPackage = token.getParentPackage();
-                IModule module = this;
-                if(nature != null){
-                	IModule mod = nature.getAstManager().getModule(parentPackage, nature, true);
-                	if(mod != null){
-                		module = mod;
-                	}
-                }
-                //line, col
-                return new Definition(def.o1, def.o2, tok, a, scopeVisitor.scope, module);
             }
         }
         
@@ -483,6 +639,9 @@ public class SourceModule extends AbstractModule {
      * @return if the file we have is the same file in the cache.
      */
     public boolean isSynched() {
+        if(this.file == null && TESTING){
+            return true; //when testing we can have a source module without a file
+        }
         return this.file.lastModified() == this.lastModified;
     }
     
@@ -545,4 +704,8 @@ public class SourceModule extends AbstractModule {
         
         return REF.getFileAbsolutePath(file).equals(REF.getFileAbsolutePath(m.file)) && this.name.equals(m.name); 
     }
+
+	public void setName(String n) {
+		this.name = n;
+	}
 }
\ No newline at end of file
