@@ -21,7 +21,6 @@ import java.util.StringTokenizer;
 import org.eclipse.core.runtime.CoreException;
 import org.eclipse.core.runtime.IStatus;
 import org.python.pydev.core.IPythonNature;
-import org.python.pydev.core.REF;
 import org.python.pydev.core.Tuple;
 import org.python.pydev.editor.actions.refactoring.PyRefactorAction.Operation;
 import org.python.pydev.editor.codecompletion.PyCodeCompletion;
@@ -45,7 +44,7 @@ public abstract class AbstractShell {
     public static final int OTHERS_SHELL = 2;
     public static final int COMPLETION_SHELL = 1;
     protected static final int DEFAULT_SLEEP_BETWEEN_ATTEMPTS = 1000;
-    protected static final int DEBUG_SHELL = 2;
+    protected static final int DEBUG_SHELL = -1;
     
     /**
      * Determines if we are already in a method that starts the shell
@@ -130,7 +129,7 @@ public abstract class AbstractShell {
 	            
 	            Map<Integer,AbstractShell> rel = (Map<Integer, AbstractShell>) iter.next();
 	            if(rel != null){
-	                for (Iterator iter2 = rel.values().iterator(); iter.hasNext();) {
+	                for (Iterator iter2 = rel.values().iterator(); iter2.hasNext();) {
 	                    AbstractShell element = (AbstractShell) iter2.next();
 	                    if(element != null){
 	                        try {
@@ -440,7 +439,10 @@ public abstract class AbstractShell {
         //will print things if we are debugging or just get it (and do nothing except emptying it)
         stdReader = new ThreadStreamReader(process.getInputStream());
         errReader = new ThreadStreamReader(process.getErrorStream());
-
+        
+        stdReader.setName("Shell reader (stdout)");
+        errReader.setName("Shell reader (stderr)");
+        
         stdReader.start();
         errReader.start();
     }
@@ -698,52 +700,6 @@ public abstract class AbstractShell {
         }
     }
 
-    public synchronized void sendGoToDirMsg(File file) {
-        while(isInOperation){
-            sleepALittle(100);
-        }
-        isInOperation = true;
-        try {
-            if (finishedForGood) {
-                throw new RuntimeException("Shells are already finished for good, so, it is an invalid state to try to change the shell dir.");
-            }
-            checkShell();
-
-            try {
-                if (file.isDirectory() == false) {
-                    file = file.getParentFile();
-                }
-
-                String str = REF.getFileAbsolutePath(file);
-                str = URLEncoder.encode(str, ENCODING_UTF_8);
-                this.write("@@CHANGE_DIR:" + str + "END@@");
-                this.read(); //this should be the ok message...
-
-            } catch (IOException e) {
-                try {
-                    restartShell();
-                } catch (Exception e1) {
-
-                }
-                PydevPlugin.log(IStatus.ERROR, "ERROR sending go to dir msg.", e);
-            }
-        } finally {
-            isInOperation = false;
-        }
-    }
-
-    private synchronized void checkShell() {
-        try {
-            if (this.socketToWrite == null || !this.socketToWrite.isBound() || 
-                this.socketToRead == null  || !this.socketToRead.isBound()  ||
-                this.serverSocket == null  || !this.serverSocket.isBound()) {
-                restartShell();
-            }
-        } catch (Exception e) {
-            // ok
-        }
-    }
-
 
     /**
      * @return list with tuples: new String[]{token, description}
