@@ -3,7 +3,6 @@ package org.python.pydev.plugin;
 import java.io.File;
 import java.io.FileFilter;
 import java.io.FileInputStream;
-import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.ObjectInputStream;
@@ -16,9 +15,11 @@ import java.util.ResourceBundle;
 
 import org.eclipse.core.resources.IContainer;
 import org.eclipse.core.resources.IFile;
+import org.eclipse.core.resources.IFolder;
 import org.eclipse.core.resources.IProject;
 import org.eclipse.core.resources.IProjectNature;
 import org.eclipse.core.resources.IResource;
+import org.eclipse.core.resources.IResourceVisitor;
 import org.eclipse.core.resources.IWorkspace;
 import org.eclipse.core.resources.IWorkspaceRoot;
 import org.eclipse.core.resources.ResourcesPlugin;
@@ -35,11 +36,18 @@ import org.eclipse.core.runtime.jobs.Job;
 import org.eclipse.jface.preference.IPreferenceStore;
 import org.eclipse.jface.text.templates.ContextTypeRegistry;
 import org.eclipse.jface.text.templates.persistence.TemplateStore;
+import org.eclipse.jface.window.Window;
+import org.eclipse.swt.widgets.Display;
+import org.eclipse.swt.widgets.FileDialog;
+import org.eclipse.swt.widgets.Shell;
 import org.eclipse.ui.IEditorInput;
 import org.eclipse.ui.IEditorPart;
 import org.eclipse.ui.IWorkbench;
 import org.eclipse.ui.IWorkbenchPage;
 import org.eclipse.ui.IWorkbenchWindow;
+import org.eclipse.ui.PartInitException;
+import org.eclipse.ui.PlatformUI;
+import org.eclipse.ui.dialogs.ElementListSelectionDialog;
 import org.eclipse.ui.editors.text.EditorsUI;
 import org.eclipse.ui.editors.text.templates.ContributionContextTypeRegistry;
 import org.eclipse.ui.editors.text.templates.ContributionTemplateStore;
@@ -54,37 +62,83 @@ import org.python.pydev.core.ICallback;
 import org.python.pydev.core.IInterpreterManager;
 import org.python.pydev.core.IPythonNature;
 import org.python.pydev.core.REF;
+import org.python.pydev.core.Tuple;
+import org.python.pydev.core.bundle.BundleInfo;
+import org.python.pydev.core.bundle.IBundleInfo;
+import org.python.pydev.core.bundle.ImageCache;
+import org.python.pydev.core.log.Log;
 import org.python.pydev.editor.PyEdit;
-import org.python.pydev.editor.codecompletion.revisited.SystemModulesManager;
+import org.python.pydev.editor.codecompletion.revisited.PythonPathHelper;
 import org.python.pydev.editor.codecompletion.shell.AbstractShell;
 import org.python.pydev.editor.templates.PyContextType;
 import org.python.pydev.plugin.nature.PythonNature;
+import org.python.pydev.plugin.nature.SystemPythonNature;
 import org.python.pydev.pyunit.ITestRunListener;
 import org.python.pydev.pyunit.PyUnitTestRunner;
-import org.python.pydev.ui.ImageCache;
 import org.python.pydev.ui.interpreters.JythonInterpreterManager;
 import org.python.pydev.ui.interpreters.PythonInterpreterManager;
 
+
 /**
  * The main plugin class - initialized on startup - has resource bundle for internationalization - has preferences
  */
 public class PydevPlugin extends AbstractUIPlugin implements Preferences.IPropertyChangeListener {
+	private static boolean DEBUG = false;
+	
+	
+    // ----------------- SINGLETON THINGS -----------------------------
+    public static IBundleInfo info;
+    public static IBundleInfo getBundleInfo(){
+        if(PydevPlugin.info == null){
+            PydevPlugin.info = new BundleInfo(PydevPlugin.getDefault().getBundle());
+        }
+        return PydevPlugin.info;
+    }
+    public static void setBundleInfo(IBundleInfo b){
+        PydevPlugin.info = b;
+    }
+    // ----------------- END BUNDLE INFO THINGS --------------------------
 	
     private static IInterpreterManager pythonInterpreterManager;
-    private static IInterpreterManager jythonInterpreterManager;
     public static void setPythonInterpreterManager(IInterpreterManager interpreterManager) {
         PydevPlugin.pythonInterpreterManager = interpreterManager;
     }
     public static IInterpreterManager getPythonInterpreterManager() {
-        return pythonInterpreterManager;
+        return getPythonInterpreterManager(false);
+    }
+    public static IInterpreterManager getPythonInterpreterManager(boolean haltOnStub) {
+    	while(pythonInterpreterManager instanceof StubInterpreterManager){ //this happens during initialization
+    		try {
+    			Thread.sleep(100);
+    		} catch (Exception e) {
+    			e.printStackTrace();
+    		}
+    	}
+    	return pythonInterpreterManager;
     }
 
+    
+    
+    
+    private static IInterpreterManager jythonInterpreterManager;
     public static void setJythonInterpreterManager(IInterpreterManager interpreterManager) {
         PydevPlugin.jythonInterpreterManager = interpreterManager;
     }
     public static IInterpreterManager getJythonInterpreterManager() {
+    	return getJythonInterpreterManager(false);
+    }
+    public static IInterpreterManager getJythonInterpreterManager(boolean haltOnStub) {
+    	while(jythonInterpreterManager instanceof StubInterpreterManager){ //this happens during initialization
+			try {
+				Thread.sleep(100);
+			} catch (Exception e) {
+				e.printStackTrace();
+			}
+    	}
         return jythonInterpreterManager;
     }
+    // ----------------- END SINGLETON THINGS --------------------------
+
     /**
      * returns the interpreter manager for a given nature
      * @param nature the nature from where we want to get the associated interpreter manager
@@ -106,7 +160,7 @@ public class PydevPlugin extends AbstractUIPlugin implements Preferences.IProper
     
     
     private static PydevPlugin plugin; //The shared instance.
-    
+
     private ResourceBundle resourceBundle; //Resource bundle.
 
     /** The template store. */
@@ -136,27 +190,36 @@ public class PydevPlugin extends AbstractUIPlugin implements Preferences.IProper
         } catch (MissingResourceException x) {
             resourceBundle = null;
         }
-        Preferences preferences = plugin.getPluginPreferences();
+        final Preferences preferences = plugin.getPluginPreferences();
         preferences.addPropertyChangeListener(this);
-        setPythonInterpreterManager(new PythonInterpreterManager(preferences));
-        setJythonInterpreterManager(new JythonInterpreterManager(preferences));
+        
+        //set them temporarily
+        setPythonInterpreterManager(new StubInterpreterManager(true));
+        setJythonInterpreterManager(new StubInterpreterManager(false));
         
 
         //restore the nature for all python projects
         new Job("PyDev: Restoring projects python nature"){
 
             protected IStatus run(IProgressMonitor monitor) {
-                IProject[] projects = getWorkspace().getRoot().getProjects();
-                for (int i = 0; i < projects.length; i++) {
-                    IProject project = projects[i];
-                    try {
-                        if (project.isOpen() && project.hasNature(PythonNature.PYTHON_NATURE_ID)) {
-                            PythonNature.addNature(project, monitor);
-                        }
-                    } catch (Exception e) {
-                        PydevPlugin.log(e);
-                    }
-                }
+            	try{
+	            	setPythonInterpreterManager(new PythonInterpreterManager(preferences));
+	            	setJythonInterpreterManager(new JythonInterpreterManager(preferences));
+	            	
+	                IProject[] projects = getWorkspace().getRoot().getProjects();
+	                for (int i = 0; i < projects.length; i++) {
+	                    IProject project = projects[i];
+	                    try {
+	                        if (project.isOpen() && project.hasNature(PythonNature.PYTHON_NATURE_ID)) {
+	                            PythonNature.addNature(project, monitor);
+	                        }
+	                    } catch (Exception e) {
+	                        PydevPlugin.log(e);
+	                    }
+	                }
+            	}catch(Throwable t){
+            		t.printStackTrace();
+            	}
                 return Status.OK_STATUS;
             }
             
@@ -164,6 +227,22 @@ public class PydevPlugin extends AbstractUIPlugin implements Preferences.IProper
         
     }
     
+    public static boolean isPythonInterpreterInitialized() {
+    	IInterpreterManager pythonInterpreterManager2 = getPythonInterpreterManager();
+    	if(pythonInterpreterManager2 instanceof StubInterpreterManager){
+    		return false;
+    	}
+    	return true;
+	}
+    
+    public static boolean isJythonInterpreterInitialized() {
+    	IInterpreterManager jythonInterpreterManager2 = getJythonInterpreterManager();
+    	if(jythonInterpreterManager2 instanceof StubInterpreterManager){
+    		return false;
+    	}
+    	return true;
+    }
+    
 
     /**
      * This is called when the plugin is being stopped.
@@ -206,7 +285,7 @@ public class PydevPlugin extends AbstractUIPlugin implements Preferences.IProper
     }
 
     public static String getPluginID() {
-        return BundleInfo.getBundleInfo().getPluginID();
+        return PydevPlugin.getBundleInfo().getPluginID();
     }
 
     /**
@@ -295,7 +374,12 @@ public class PydevPlugin extends AbstractUIPlugin implements Preferences.IProper
 
     public static void logInfo(String msg) {
         IStatus s = PydevPlugin.makeStatus(IStatus.INFO, msg, null);
-        getDefault().getLog().log(s);
+        PydevPlugin plug = getDefault();
+        if(plug == null){//testing mode
+            System.out.println(msg);
+        }else{
+            plug.getLog().log(s);
+        }
     }
     
     public static CoreException log(String msg) {
@@ -305,70 +389,20 @@ public class PydevPlugin extends AbstractUIPlugin implements Preferences.IProper
         return e;
     }
 
-    /**
-     *  
-     */
-    public static IPath getPath(IPath location) {
-        IWorkspace workspace = ResourcesPlugin.getWorkspace();
-        IWorkspaceRoot root = workspace.getRoot();
-
-        IFile[] files = root.findFilesForLocation(location);
-        IContainer[] folders =  root.findContainersForLocation(location);
-        if (files.length > 0) {
-            for (int i = 0; i < files.length; i++)
-                return files[i].getProjectRelativePath();
-        } else {
-            for (int i = 0; i < folders.length; i++)
-                return folders[i].getProjectRelativePath();
-        }
-
-        return null;
-    }
-
-    public static IPath getLocationFromWorkspace(IPath path) {
-        return getLocationFromWorkspace(path, 0);
-    }
-    /**
-     * This one should only be used if the root (project) is unknown.
-     * 
-     * @see PydevPlugin.getLocation#IPath, IContainer
-     * @param path
-     * @return
-     */
-    public static IPath getLocationFromWorkspace(IPath path, int repetitions) {
-        IWorkspace workspace = ResourcesPlugin.getWorkspace();
-        IContainer root = workspace.getRoot();
-        repetitions++;
-        return getLocation(path, root, repetitions);
-    }
-
-    public static IPath getLocation(IPath path, IContainer root) {
-        return getLocation(path, root, 0);
+    public static IEditorPart doOpenEditor(IFile f, boolean activate) {
+    	if (f == null)
+    		return null;
+    	
+    	try {
+    		FileEditorInput file = new FileEditorInput(f);
+    		return openEditorInput(file);
+    		
+    	} catch (Exception e) {
+    		log(IStatus.ERROR, "Unexpected error opening path " + f.toString(), e);
+    		return null;
+    	}
     }
     
-    /**
-     * Returns the location in the filesystem for the given path
-     * 
-     * @param path
-     * @param root the path must be inside this root
-     * @return
-     */
-    public static IPath getLocation(IPath path, IContainer root, int repetitions) {
-        if(repetitions > 3){
-            return null;
-        }
-        repetitions++;
-        IResource resource = root.findMember(path);
-        IPath location = null;
-        if (resource != null) {
-            location = resource.getLocation();
-        }
-        
-        if(location == null){
-            location = getLocationFromWorkspace(path, repetitions);
-        }
-        return location;
-    }
     /**
      * Utility function that opens an editor on a given path.
      * 
@@ -379,29 +413,32 @@ public class PydevPlugin extends AbstractUIPlugin implements Preferences.IProper
             return null;
 
         try {
-            
-            IEditorInput file = createEditorInput(path);
-	        
-	        final IWorkbench workbench = plugin.getWorkbench();
-	        if(workbench == null){
-	        	throw new RuntimeException("workbench cannot be null");
-	        }
-
-	        IWorkbenchWindow activeWorkbenchWindow = workbench.getActiveWorkbenchWindow();
-	        if(activeWorkbenchWindow == null){
-	        	throw new RuntimeException("activeWorkbenchWindow cannot be null (we have to be in a ui thread for this to work)");
-	        }
-	        
-			IWorkbenchPage wp = activeWorkbenchWindow.getActivePage();
-        
-            // File is inside the workspace
-            return IDE.openEditor(wp, file, PyEdit.EDITOR_ID);
+    		IEditorInput file = createEditorInput(path);
+	        return openEditorInput(file);
             
         } catch (Exception e) {
             log(IStatus.ERROR, "Unexpected error opening path " + path.toString(), e);
             return null;
         }
     }
+    
+    
+	private static IEditorPart openEditorInput(IEditorInput file) throws PartInitException {
+		final IWorkbench workbench = plugin.getWorkbench();
+		if(workbench == null){
+			throw new RuntimeException("workbench cannot be null");
+		}
+
+		IWorkbenchWindow activeWorkbenchWindow = workbench.getActiveWorkbenchWindow();
+		if(activeWorkbenchWindow == null){
+			throw new RuntimeException("activeWorkbenchWindow cannot be null (we have to be in a ui thread for this to work)");
+		}
+		
+		IWorkbenchPage wp = activeWorkbenchWindow.getActivePage();
+      
+		// File is inside the workspace
+		return IDE.openEditor(wp, file, PyEdit.EDITOR_ID);
+	}
 
     
     
@@ -411,28 +448,135 @@ public class PydevPlugin extends AbstractUIPlugin implements Preferences.IProper
 //  ===================== ALL BELOW IS COPIED FROM org.eclipse.ui.internal.editors.text.OpenExternalFileAction
 //  =====================
 
+    public static IEditorInput createEditorInput(IPath path) {
+        return createEditorInput(path, true);
+    }
     /**
      * @param path
      * @return
      */
-    public static IEditorInput createEditorInput(IPath path) {
-        IEditorInput edInput;
+    private static IEditorInput createEditorInput(IPath path, boolean askIfDoesNotExist) {
+        IEditorInput edInput = null;
         IWorkspace w = ResourcesPlugin.getWorkspace();      
-        IFile file = w.getRoot().getFileForLocation(path);
-        if (file == null  || !file.exists()){
+
+        //let's start with the 'easy' way
+    	IFile fileForLocation = w.getRoot().getFileForLocation(path);
+    	if(fileForLocation != null){
+    		return new FileEditorInput(fileForLocation);
+    	}
+
+        
+        
+        IFile files[] = w.getRoot().findFilesForLocation(path);
+        if (files == null  || files.length == 0 || !files[0].exists()){
             //it is probably an external file
-            File file2 = path.toFile();
-            edInput = createEditorInput(file2);
-        }else{
-            edInput = new FileEditorInput(file);
+            File systemFile = path.toFile();
+            if(systemFile.exists()){
+                edInput = createEditorInput(systemFile);
+                
+            }else if(askIfDoesNotExist){
+                //this is the last resort... First we'll try to check for a 'good' match,
+                //and if there's more than one we'll ask it to the user
+                List<IFile> likelyFiles = getLikelyFiles(path, w);
+                IFile iFile = selectWorkspaceFile(likelyFiles.toArray(new IFile[0]));
+                if(iFile != null){
+                    return new FileEditorInput(iFile);
+                }
+                
+                //ok, ask the user for any file in the computer
+                IEditorInput input = selectFilesystemFileForPath(path);
+                if(input != null){
+                    return input;
+                }
+            }
+        }else{ //file exists
+            edInput = doFileEditorInput(selectWorkspaceFile(files));
         }
         return edInput;
     }
 
+    private static IEditorInput doFileEditorInput(IFile file) {
+        if(file == null){
+            return null;
+        }
+        return new FileEditorInput(file);
+    }
+    /**
+     * This is the last resort... pointing to some filesystem file to get the editor for some path.
+     */
+    private static IEditorInput selectFilesystemFileForPath(final IPath path) {
+        final List<String> l = new ArrayList<String>();
+        Runnable r = new Runnable(){
+
+            public void run() {
+                Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
+                FileDialog dialog = new FileDialog(shell);
+                dialog.setText(path+" - select correspondent filesystem file.");
+                dialog.setFilterExtensions(new String[]{"*.py; *.pyw"});
+                String string = dialog.open();
+                if(string != null){
+                    l.add(string);
+                }
+            }
+        };
+        if(Display.getCurrent() == null){ //not ui-thread
+            Display.getDefault().syncExec(r);
+        }else{
+            r.run();
+        }
+        if(l.size() > 0){
+            String fileAbsolutePath = REF.getFileAbsolutePath(l.get(0));
+            return new PydevFileEditorInput(new File(fileAbsolutePath));
+        }
+        return null;
+    }
+    
+    /**
+     * This method will pass all the files in the workspace and check if there's a file that might
+     * be a match to some path (use only as an almost 'last-resort').
+     */
+    private static List<IFile> getLikelyFiles(IPath path, IWorkspace w) {
+        List<IFile> ret = new ArrayList<IFile>();
+        try {
+            IResource[] resources = w.getRoot().members();
+            getLikelyFiles(path, ret, resources);
+        } catch (CoreException e) {
+            Log.log(e);
+        }
+        return ret;
+    }
+    
+    /**
+     * Used to recursively get the likely files given the first set of containers
+     */
+    private static void getLikelyFiles(IPath path, List<IFile> ret, IResource[] resources) throws CoreException {
+        String strPath = path.removeFileExtension().lastSegment().toLowerCase(); //this will return something as 'foo'
+        
+        for (IResource resource : resources) {
+            if(resource instanceof IFile){
+                IFile f = (IFile) resource;
+                
+                if(PythonPathHelper.isValidSourceFile(f)){
+                    if(resource.getFullPath().removeFileExtension().lastSegment().toLowerCase().equals(strPath)){ 
+                        ret.add((IFile) resource);
+                    }
+                }
+            }else if(resource instanceof IContainer){
+                getLikelyFiles(path, ret, ((IContainer)resource).members());
+            }
+        }
+    }
+    
     private static IEditorInput createEditorInput(File file) {
         IFile[] workspaceFile= getWorkspaceFile(file);
-        if (workspaceFile != null && workspaceFile.length > 0)
-            return new FileEditorInput(workspaceFile[0]);
+        if (workspaceFile != null && workspaceFile.length > 0){
+            IFile file2 = selectWorkspaceFile(workspaceFile);
+            if(file2 != null){
+                return new FileEditorInput(file2);
+            }else{
+                return new FileEditorInput(workspaceFile[0]);
+            }
+        }
         return new PydevFileEditorInput(file);
     }
 
@@ -446,10 +590,6 @@ public class PydevPlugin extends AbstractUIPlugin implements Preferences.IProper
         }
         
         return files;
-        //we are out of the loop from the interface when this is called
-//        if (files.length == 1)
-//            return files[0];
-//        return selectWorkspaceFile(files);
     }
     
 
@@ -465,15 +605,39 @@ public class PydevPlugin extends AbstractUIPlugin implements Preferences.IProper
         }
         return (IFile[])existentFiles.toArray(new IFile[existentFiles.size()]);
     }
-//    private IFile selectWorkspaceFile(IFile[] files) {
-//        ElementListSelectionDialog dialog= new ElementListSelectionDialog(fWindow.getShell(), new FileLabelProvider());
-//        dialog.setElements(files);
-//        dialog.setTitle(TextEditorMessages.OpenExternalFileAction_title_selectWorkspaceFile);
-//        dialog.setMessage(TextEditorMessages.OpenExternalFileAction_message_fileLinkedToMultiple);
-//        if (dialog.open() == Window.OK)
-//            return (IFile) dialog.getFirstResult();
-//        return null;
-//    }
+    
+    private static IFile selectWorkspaceFile(final IFile[] files) {
+        if(files.length == 0){
+            return null;
+        }
+        if(files.length == 1){
+            return files[0];
+        }
+        final List<IFile> selected = new ArrayList<IFile>();
+        
+        Runnable r = new Runnable(){
+            public void run() {
+                Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
+                ElementListSelectionDialog dialog= new ElementListSelectionDialog(shell, new PyFileLabelProvider());
+                dialog.setElements(files);
+                dialog.setTitle("Select Workspace File");
+                dialog.setMessage("File may be matched to multiple files in the workspace.");
+                if (dialog.open() == Window.OK){
+                    selected.add((IFile) dialog.getFirstResult());
+                }
+            }
+            
+        };
+        if(Display.getCurrent() == null){ //not ui-thread
+            Display.getDefault().syncExec(r);
+        }else{
+            r.run();
+        }
+        if(selected.size() > 0){
+            return selected.get(0);
+        }
+        return null;
+    }
 
     
 //  =====================
@@ -532,11 +696,11 @@ public class PydevPlugin extends AbstractUIPlugin implements Preferences.IProper
      * @throws CoreException
      */
     public static File getRelativePath(IPath relative) throws CoreException {
-        return BundleInfo.getBundleInfo().getRelativePath(relative);
+        return PydevPlugin.getBundleInfo().getRelativePath(relative);
     }
     
     public static ImageCache getImageCache(){
-        return BundleInfo.getBundleInfo().getImageCache();
+        return PydevPlugin.getBundleInfo().getImageCache();
     }
     
 
@@ -549,14 +713,28 @@ public class PydevPlugin extends AbstractUIPlugin implements Preferences.IProper
     }
 
     /**
-     * Returns the directories and python files in a list.
-     * 
-     * @param file
-     * @return tuple with files in pos 0 and folders in pos 1
+     * @return All the IFiles below the current folder that are python files (does not check if it has an __init__ path)
      */
-    public static List<File>[] getPyFilesBelow(File file, IProgressMonitor monitor, final boolean includeDirs) {
-        return getPyFilesBelow(file, monitor, true, true);
+    public static List<IFile> getAllIFilesBelow(IFolder member) {
+    	final ArrayList<IFile> ret = new ArrayList<IFile>();
+    	try {
+			member.accept(new IResourceVisitor(){
+
+				public boolean visit(IResource resource) {
+					if(resource instanceof IFile){
+						ret.add((IFile) resource);
+						return false; //has no members
+					}
+					return true;
+				}
+				
+			});
+		} catch (CoreException e) {
+			throw new RuntimeException(e);
+		}
+    	return ret;
     }
+    
     /**
      * Returns the directories and python files in a list.
      * 
@@ -564,18 +742,39 @@ public class PydevPlugin extends AbstractUIPlugin implements Preferences.IProper
      * @return tuple with files in pos 0 and folders in pos 1
      */
     public static List<File>[] getPyFilesBelow(File file, IProgressMonitor monitor, final boolean includeDirs, boolean checkHasInit) {
-        FileFilter filter = new FileFilter() {
+        FileFilter filter = getPyFilesFileFilter(includeDirs);
+        return getPyFilesBelow(file, filter, monitor, true, checkHasInit);
+    }
+    
+    /**
+     * @param includeDirs determines if we can include subdirectories
+     * @return a file filter only for python files (and other dirs if specified)
+     */
+	public static FileFilter getPyFilesFileFilter(final boolean includeDirs) {
+		return new FileFilter() {
     
             public boolean accept(File pathname) {
-                if (includeDirs)
-                    return pathname.isDirectory() || pathname.toString().endsWith(".py");
-                else
-                    return pathname.isDirectory() == false && pathname.toString().endsWith(".py");
+                if (includeDirs){
+                	if(pathname.isDirectory()){
+                		return true;
+                	}
+                	if(PythonPathHelper.isValidSourceFile(pathname.toString())){
+                		return true;
+                	}
+                	return false;
+                }else{
+                	if(pathname.isDirectory()){
+                		return false;
+                	}
+                	if(PythonPathHelper.isValidSourceFile(pathname.toString())){
+                		return true;
+                	}
+                	return false;
+                }
             }
     
         };
-        return getPyFilesBelow(file, filter, monitor, true, checkHasInit);
-    }
+	}
 
 
     public static List<File>[] getPyFilesBelow(File file, FileFilter filter, IProgressMonitor monitor, boolean checkHasInit) {
@@ -592,6 +791,7 @@ public class PydevPlugin extends AbstractUIPlugin implements Preferences.IProper
      * @param addSubFolders: indicates if sub-folders should be added
      * @return tuple with files in pos 0 and folders in pos 1
      */
+    @SuppressWarnings("unchecked")
     private static List<File>[] getPyFilesBelow(File file, FileFilter filter, IProgressMonitor monitor, boolean addSubFolders, int level, boolean checkHasInit) {
         if (monitor == null) {
             monitor = new NullProgressMonitor();
@@ -622,8 +822,8 @@ public class PydevPlugin extends AbstractUIPlugin implements Preferences.IProper
 	                    monitor.worked(1);
 	                    monitor.setTaskName("Found:" + file2.toString());
 	                    
-                        if (checkHasInit){
-    	                    if(file2.getName().equals("__init__.py")){
+                        if (checkHasInit && hasInit == false){
+    	                    if(PythonPathHelper.isValidInitFile(file2.getName())){
     	                        hasInit = true;
     	                    }
                         }
@@ -669,7 +869,8 @@ public class PydevPlugin extends AbstractUIPlugin implements Preferences.IProper
     private List listeners = new ArrayList();
 
 
-	public void addTestListener(ITestRunListener listener) {
+	@SuppressWarnings("unchecked")
+    public void addTestListener(ITestRunListener listener) {
 		listeners.add(listener);
 	}
 	
@@ -712,6 +913,71 @@ public class PydevPlugin extends AbstractUIPlugin implements Preferences.IProper
 			each.testFailed(klass, methodName, trace);
 		}
 	}
+    /**
+     * @param file the file we want to get info on.
+     * @return a tuple with the pythonnature to be used and the name of the module represented by the file in that scenario.
+     */
+    public static Tuple<SystemPythonNature, String> getInfoForFile(File file){
+        String modName = null;
+        IInterpreterManager pythonInterpreterManager = getPythonInterpreterManager();
+        IInterpreterManager jythonInterpreterManager = getJythonInterpreterManager();
+    
+        SystemPythonNature systemPythonNature = new SystemPythonNature(pythonInterpreterManager);
+        SystemPythonNature pySystemPythonNature = systemPythonNature;
+        SystemPythonNature jySystemPythonNature = null;
+        try {
+            modName = systemPythonNature.resolveModule(file);
+        } catch (Exception e) {
+            // that's ok
+        }
+        if(modName == null){
+            systemPythonNature = new SystemPythonNature(jythonInterpreterManager);
+            jySystemPythonNature = systemPythonNature;
+            try {
+                modName = systemPythonNature.resolveModule(file);
+            } catch (Exception e) {
+                // that's ok
+            }
+        }
+        if(modName != null){
+            return new Tuple<SystemPythonNature, String>(systemPythonNature, modName);
+        }else{
+            //unable to discover it
+            try {
+                // the default one is python (actually, this should never happen, but who knows)
+                pythonInterpreterManager.getDefaultInterpreter();
+                modName = getModNameFromFile(file);
+                return new Tuple<SystemPythonNature, String>(pySystemPythonNature, modName);
+            } catch (Exception e) {
+                //the python interpreter manager is not valid or not configured
+                try {
+                    // the default one is jython
+                    jythonInterpreterManager.getDefaultInterpreter();
+                    modName = getModNameFromFile(file);
+                    return new Tuple<SystemPythonNature, String>(jySystemPythonNature, modName);
+                } catch (Exception e1) {
+                    // ok, nothing to do about it, no interpreter is configured
+                    return null;
+                }
+            }
+        }
+    }
+    
+    /**
+     * This is the last resort (should not be used anywhere else).
+     */
+    private static String getModNameFromFile(File file) {
+        if(file == null){
+            return null;
+        }
+        String name = file.getName();
+        int i = name.indexOf('.');
+        if (i != -1){
+            return name.substring(0, i);
+        }
+        return name;
+    }
+
     /**
      * @return a preference store that has the pydev preference store and the default editors text store
      */
