@@ -33,8 +33,10 @@ import org.python.pydev.builder.PyDevBuilderPrefPage;
 import org.python.pydev.core.ExtensionHelper;
 import org.python.pydev.core.ICodeCompletionASTManager;
 import org.python.pydev.core.IInterpreterManager;
+import org.python.pydev.core.IModule;
 import org.python.pydev.core.IPythonNature;
 import org.python.pydev.core.IPythonPathNature;
+import org.python.pydev.core.IToken;
 import org.python.pydev.core.REF;
 import org.python.pydev.editor.codecompletion.revisited.ASTManager;
 import org.python.pydev.plugin.PydevPlugin;
@@ -84,12 +86,27 @@ public class PythonNature implements IPythonNature {
     /**
      * We have to know if it has already been initialized.
      */
-    private boolean initialized;
+    private boolean initializationStarted;
+    
+    /**
+     * We have to know if it has already been initialized.
+     */
+    private boolean initializationFinished;
 
     /**
      * Manages pythonpath things
      */
     private IPythonPathNature pythonPathNature = new PythonPathNature();
+    
+    /**
+     * This is used to keep the builtin completions
+     */
+    private IToken[] builtinCompletions;
+    
+    /**
+     * This is used to keep the builtin module
+     */
+    private IModule builtinMod;
 
     /**
      * This method is called only when the project has the nature added..
@@ -210,8 +227,8 @@ public class PythonNature implements IPythonNature {
      */
     private void init() {
         final PythonNature nature = this;
-        if (initialized == false) {
-            initialized = true;
+        if (initializationStarted == false) {
+            initializationStarted = true;
             
             Job codeCompletionLoadJob = new Job("Pydev code completion") {
 
@@ -258,6 +275,7 @@ public class PythonNature implements IPythonNature {
                             PydevPlugin.log(e);
                         }
                     }
+                    initializationFinished = true;
                     jobProgressComunicator.done();
                     return Status.OK_STATUS;
                 }
@@ -306,6 +324,7 @@ public class PythonNature implements IPythonNature {
     
 	public void rebuildPath(String defaultSelectedInterpreter, IProgressMonitor monitor) {
 		try {
+			clearCaches();
 			String paths = this.pythonPathNature.getOnlyProjectPythonPathStr();
 			this.rebuildPath(defaultSelectedInterpreter, paths);
 		} catch (Exception e) {
@@ -313,6 +332,7 @@ public class PythonNature implements IPythonNature {
 		}
 	}
 
+
     /**
      * This method is called whenever the pythonpath for the project with this nature is changed. 
      */
@@ -350,7 +370,8 @@ public class PythonNature implements IPythonNature {
                 } catch (Throwable e) {
                     PydevPlugin.log(e);
                 }
-                                                
+
+                initializationFinished = true;
                 //end task
                 jobProgressComunicator.done();
                 return Status.OK_STATUS;
@@ -363,7 +384,24 @@ public class PythonNature implements IPythonNature {
     /**
      * @return Returns the completionsCache.
      */
-    public ICodeCompletionASTManager getAstManager() {    
+    public ICodeCompletionASTManager getAstManager() {
+        if(astManager == null){
+        	//this is needed because it may not be restarted already...
+        	//also, this will only happen when initializing eclipse with some editors already open
+        	
+        	for(int i=0; i<10 && astManager == null && !initializationFinished; i++){ //we will wait 10 seconds for it
+        		try {
+					Thread.sleep(1000);
+        		} catch (InterruptedException e) {
+                    //ignore
+				} catch (Exception e) {
+					e.printStackTrace();
+				}
+        	}
+        	//next time we won't wait as long.
+        	initializationFinished = true;
+        }
+
         return astManager;
     }
     
@@ -399,6 +437,13 @@ public class PythonNature implements IPythonNature {
     	return natures;
     }
     
+    public static PythonNature getPythonNature(IResource resource) {
+    	if(resource == null){
+    		return null;
+    	}
+    	return getPythonNature(resource.getProject());
+    	
+    }
     /**
      * @param project the project we want to know about (if it is null, null is returned)
      * @return the python nature for a project (or null if it does not exist for the project)
@@ -406,11 +451,9 @@ public class PythonNature implements IPythonNature {
     public static PythonNature getPythonNature(IProject project) {
         if(project != null){
             try {
-                if(project.hasNature(PYTHON_NATURE_ID)){
-	                IProjectNature n = project.getNature(PYTHON_NATURE_ID);
-	                if(n instanceof PythonNature){
-	                    return (PythonNature) n;
-	                }
+                IProjectNature n = project.getNature(PYTHON_NATURE_ID);
+                if(n instanceof PythonNature){
+                    return (PythonNature) n;
                 }
             } catch (CoreException e) {
                 PydevPlugin.logInfo(e);
@@ -419,18 +462,26 @@ public class PythonNature implements IPythonNature {
         return null;
     }
 
+    /**
+     * Stores the version (the actual version is as a persistent property in the project).
+     * This is so that we don't have a runtime penalty for it.
+     */
+    private String persistentProperty = null;
+    
     /**
      * @return the python version for the project
      * @throws CoreException 
      */
     public String getVersion() throws CoreException {
         if(project != null){
-            String persistentProperty = project.getPersistentProperty(PYTHON_PROJECT_VERSION);
-            if(persistentProperty == null){ //there is no such property set (let's set it to the default
-                String defaultVersion = getDefaultVersion();
-                setVersion(defaultVersion);
-                persistentProperty = defaultVersion;
-            }
+        	if(persistentProperty == null){
+	            persistentProperty = project.getPersistentProperty(PYTHON_PROJECT_VERSION);
+	            if(persistentProperty == null){ //there is no such property set (let's set it to the default)
+	                String defaultVersion = getDefaultVersion();
+	                setVersion(defaultVersion);
+	                persistentProperty = defaultVersion;
+	            }
+        	}
             return persistentProperty;
         }
         return null;
@@ -440,7 +491,9 @@ public class PythonNature implements IPythonNature {
      * @throws CoreException 
      */
     public void setVersion(String version) throws CoreException{
+		clearCaches();
         if(project != null){
+        	this.persistentProperty = version;
             project.setPersistentProperty(PYTHON_PROJECT_VERSION, version);
         }
     }
@@ -449,8 +502,12 @@ public class PythonNature implements IPythonNature {
         return PYTHON_VERSION_2_4;
     }
 
+    
     public boolean isJython() throws CoreException {
-        return getVersion().equals(JYTHON_VERSION_2_1);
+        if(isJython == null){
+            isJython = getVersion().equals(JYTHON_VERSION_2_1);
+        }
+        return isJython;
     }
 
     public boolean isPython() throws CoreException {
@@ -485,7 +542,17 @@ public class PythonNature implements IPythonNature {
         throw new RuntimeException("Unable to get the id to which this nature is related");
     }
 
-    /**
+
+	private static String getModuleName(String moduleName, PythonNature nature, String file) {
+		ICodeCompletionASTManager astManager = nature.getAstManager();
+		
+		if(astManager != null){
+		    moduleName = astManager.getModulesManager().resolveModule(file);
+		}
+		return moduleName;
+	}
+	
+	/**
      * @param resource the resource we want to get the name from
      * @return the name of the module in the environment
      */
@@ -495,15 +562,25 @@ public class PythonNature implements IPythonNature {
         
         if(nature != null){
             String file = PydevPlugin.getIResourceOSString(resource);
-            ICodeCompletionASTManager astManager = nature.getAstManager();
-            
-            if(astManager != null){
-                moduleName = astManager.getModulesManager().resolveModule(file);
-            }
+            moduleName = getModuleName(moduleName, nature, file);
         }
         return moduleName;
     }
     
+    /**
+     * @param resource the resource we want to get the name from
+     * @return the name of the module in the environment
+     */
+    public static String getModuleNameForResource(String absPath, IProject p) {
+    	String moduleName = null;
+    	PythonNature nature = getPythonNature(p);
+    	
+    	if(nature != null){
+    		moduleName = getModuleName(moduleName, nature, absPath);
+    	}
+    	return moduleName;
+    }
+    
     /**
      * @param resource the resource we want info on
      * @return whether the passed resource is in the pythonpath or not (it must be in a source folder for that).
@@ -512,6 +589,10 @@ public class PythonNature implements IPythonNature {
     	return getModuleNameForResource(resource) != null; 
     }
     
+    public static boolean isResourceInPythonpath(String absPath, IProject p) {
+    	return getModuleNameForResource(absPath, p) != null; 
+    }
+    
     public String resolveModule(File file) {
     	String moduleName = null;
     	
@@ -539,6 +620,52 @@ public class PythonNature implements IPythonNature {
         
     }
 
+    
+    // ------------------------------------------------------------------------------------------ LOCAL CACHES
+    public void clearCaches() {
+        this.builtinCompletions = null; //when the interpreter is changed, we have to reset it
+        this.builtinMod = null;
+        this.isJython = null;
+    }
+    
+    Boolean isJython = null; //cache
+    
+	public void setBuiltinCompletions(IToken[] comps) {
+		this.builtinCompletions = comps;
+	}
+
+	public IToken[] getBuiltinCompletions() {
+		return builtinCompletions;
+	}
+
+	public IModule getBuiltinMod() {
+		return builtinMod;
+	}
+
+	public void setBuiltinMod(IModule mod) {
+		this.builtinMod = mod;
+	}
+
+    public static List<IPythonNature> getPythonNaturesRelatedTo(int relatedTo) {
+        ArrayList<IPythonNature> ret = new ArrayList<IPythonNature>();
+        IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
+        IProject[] projects = root.getProjects();
+        for (IProject project : projects) {
+            PythonNature nature = getPythonNature(project);
+            try {
+                if(nature != null){
+                    if(nature.getRelatedId() == relatedTo){
+                        ret.add(nature);
+                    }
+                }
+            } catch (CoreException e) {
+                throw new RuntimeException(e);
+            }
+        }
+        
+        return ret;
+    }
+
 }
 
 
