@@ -4,8 +4,8 @@ import java.io.File;
 
 import org.eclipse.jface.viewers.LabelProvider;
 import org.eclipse.swt.graphics.Image;
+import org.python.pydev.core.bundle.ImageCache;
 import org.python.pydev.plugin.PydevPlugin;
-import org.python.pydev.ui.ImageCache;
 import org.python.pydev.ui.UIConstants;
 
 public class FileTreeLabelProvider extends LabelProvider {
