@@ -370,7 +370,7 @@ public abstract class AbstractInterpreterEditor extends PythonListEditor {
                             String filePath = dialog.open();
                             if(filePath != null){
                                 info.libs.add(filePath);
-                                System.out.println("do add folder");
+                                changed = true;
                             }
                             
                         } else if (widget == addBtSystemJar) {
@@ -379,7 +379,7 @@ public abstract class AbstractInterpreterEditor extends PythonListEditor {
                             if(filePath != null){
                                 info.libs.add(filePath);
                                 info.dllLibs.add(filePath);
-                                System.out.println("do add jar");
+                                changed = true;
                             }
                                 
                         } else if (widget == removeBtSystemFolder) {
@@ -389,8 +389,8 @@ public abstract class AbstractInterpreterEditor extends PythonListEditor {
                                 String text = s.getText();
                                 info.libs.remove(text);
                                 info.dllLibs.remove(text);
+                                changed = true;
                             }
-                            System.out.println("do remove");
                         }
                         updateTree();
                     }
@@ -591,14 +591,14 @@ public abstract class AbstractInterpreterEditor extends PythonListEditor {
     protected void doStore() {
         String s = createList(list.getItems());
         if (s != null){
-            getPreferenceStore().setValue(getPreferenceName(), s);
+        	interpreterManager.setPersistedString(s);
         }
     }
     
     @Override
     protected void doLoad() {
         if (list != null) {
-            String s = getPreferenceStore().getString(getPreferenceName());
+        	String s = interpreterManager.getPersistedString();
             String[] array = parseString(s);
             for (int i = 0; i < array.length; i++) {
                 list.add(array[i]);
@@ -607,6 +607,9 @@ public abstract class AbstractInterpreterEditor extends PythonListEditor {
         updateTree();
     }
 
+    public String getPreferenceName(){
+    	throw new RuntimeException("The preferences should be stored/gotten from the IInterpreterManager, and not directly.");
+    }
     
     /** Overriden
      */
